@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)align.h	3.1	91/12/29	*/
+/*	SCCS Id: @(#)align.h	3.2	91/12/29	*/
 /* Copyright (c) Mike Stephenson, Izchak Miller  1991.		  */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -12,7 +12,8 @@ typedef struct	align {		/* alignment & record */
 	int		record;
 } align;
 
-#define ALIGNLIM	(5L + (moves/200L))	/* bounds for "record" */
+/* bounds for "record" -- respect initial alignments of 10 */
+#define ALIGNLIM	(10L + (moves/200L))
 
 #define A_NONE	      (-128)	/* the value range of type */
 
@@ -31,7 +32,7 @@ typedef struct	align {		/* alignment & record */
 #define AM_MASK		 7
 
 #define Amask2align(x)	((aligntyp) ((!(x)) ? A_NONE \
-			 : ((x) == AM_LAWFUL) ? A_LAWFUL : (x) - 2))
+			 : ((x) == AM_LAWFUL) ? A_LAWFUL : ((int)x) - 2))
 #define Align2amask(x)	(((x) == A_NONE) ? AM_NONE \
 			 : ((x) == A_LAWFUL) ? AM_LAWFUL : (x) + 2)
 
