@@ -1,18 +1,25 @@
-/*	SCCS Id: @(#)flag.h	3.1	93/03/18	*/
+/*	SCCS Id: @(#)flag.h	3.2	96/05/26	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
+/* If you change this file make sure you increment EDITLEVEL in   */
+/* patchlevel.h if needed.					  */
+
 #ifndef FLAG_H
 #define FLAG_H
 
-struct flag {
+struct flag {	
 #ifdef AMIFLUSH
+	boolean  altmeta;	/* use ALT keys as META */
 	boolean  amiflush;	/* kill typeahead */
 #endif
 #ifdef	MFLOPPY
 	boolean  asksavedisk;
 #endif
 	boolean  beginner;
+#ifdef MAIL
+	boolean  biff;		/* enable checking for mail */
+#endif
 #ifdef MICRO
 	boolean  BIOS;		/* use IBM or ST BIOS calls when appropriate */
 #endif
@@ -26,6 +33,9 @@ struct flag {
 	boolean  echo;		/* 1 to echo characters */
 	boolean  end_own;	/* list all own scores */
 	boolean  explore;	/* in exploration mode */
+#ifdef OPT_DISPMAP
+	boolean  fast_map;	/* use optimized, less flexible map display */
+#endif
 #define discover flags.explore
 	boolean  female;
 	boolean  friday13;	/* it's Friday the 13th */
@@ -39,8 +49,10 @@ struct flag {
 	boolean  legacy;	/* print game entry "story" */
 	boolean  lit_corridor;	/* show a dark corr as lit if it is in sight */
 	boolean  made_amulet;
+	boolean  mon_moving;	/* monsters' turn to move */
 	boolean  move;
 	boolean  mv;
+	boolean  nap;		/* `timed_delay' option for display effects */
 	boolean  news;		/* print news */
 	boolean  nopick;	/* do not pickup objects (as when running) */
 	boolean  null;		/* OK to send nulls to the terminal */
@@ -48,6 +60,7 @@ struct flag {
 #ifdef MAC
 	boolean  page_wait;	/* put up a --More-- after a page of messages */
 #endif
+	boolean  perm_invent;	/* keep full inventories up until dismissed */
 	boolean  pickup;	/* whether you pickup or move and look */
 #ifdef MAC
 	boolean  popup_dialog;	/* put queries in pop up dialogs instead of
@@ -58,6 +71,9 @@ struct flag {
 #endif
 	boolean  rest_on_space;	/* space means rest */
 	boolean  safe_dog;	/* give complete protection to the dog */
+#ifdef WIZARD
+	boolean	 sanity_check;	/* run sanity checks */
+#endif
 #ifdef EXP_ON_BOTL
 	boolean  showexp;	/* show experience points */
 #endif
@@ -70,6 +86,7 @@ struct flag {
 	boolean  standout;	/* use standout for --More-- */
 	boolean  time;		/* display elapsed 'time' */
 	boolean  tombstone;	/* print tombstone */
+	boolean  toptenwin;	/* ending list in window instead of stdout */
 #ifdef TEXTCOLOR
 	boolean  use_color;	/* use color graphics */
 	boolean  hilite_pet;	/* hilight pets on monochome displays */
@@ -77,19 +94,20 @@ struct flag {
 	boolean  verbose;	/* max battle info */
 
 	boolean  window_inited;	/* true if init_nhwindows() completed */
-	unsigned end_top, end_around;	/* describe desired score list */
+	int	 end_top, end_around;	/* describe desired score list */
 	unsigned ident;		/* social security number for each monster */
 	unsigned moonphase;
 #define NEW_MOON	0
 #define FULL_MOON	4
 	unsigned msg_history;	/* hint: # of top lines to save */
 	unsigned no_of_wizards;	/* 0, 1 or 2 (wizard and his shadow) */
-				/* reset from 2 to 1, but never to 0 */
 	unsigned run;		/* 0: h (etc), 1: H (etc), 2: fh (etc) */
 				/* 3: FH, 4: ff+, 5: ff-, 6: FF+, 7: FF- */
+	int	 djinni_count, ghost_count;	/* potion effect tuning */
 	char	 inv_order[MAXOCLASSES];
 	char	 pickup_types[MAXOCLASSES];
 	char	 end_disclose[5];	/* disclose various info upon exit */
+	char	 menu_style;	/* User interface style setting */
 #ifdef MAC_GRAPHICS_ENV
 	boolean  large_font;	/* draw in larger fonts (say, 12pt instead
 				   of 9pt) */
@@ -97,10 +115,29 @@ struct flag {
 				   as defined in the special font HackFont */
 #endif
 #ifdef AMII_GRAPHICS
-	unsigned short amii_curmap[8]; /* colormap */
+	int numcols;
+	unsigned short amii_dripens[ 20 ]; /* DrawInfo Pens currently there are 13 in v39 */
+	AMII_COLOR_TYPE amii_curmap[ AMII_MAXCOLORS ]; /* colormap */
+#endif
+#ifdef MSDOS
+	boolean	hasvga;			/* has a vga adapter                 */
+	boolean usevga;			/* use the vga adapter               */
+	boolean has8514;
+	boolean use8514;
+	boolean hasvesa;
+	boolean usevesa;
+	boolean grmode;			/* currently in graphics mode        */
+#endif
+
+#if defined(MSDOS) || defined(WIN32)
+	boolean hassound;		/* has a sound card                  */
+	boolean usesound;		/* use the sound card                */
+	boolean usepcspeaker;		/* use the pc speaker		     */
+	boolean preload_tiles;		/* preload the tiles into RAM        */
 #endif
 };
 
+
 extern NEARDATA struct flag flags;
 
 #endif /* FLAG_H */
