@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)sp_lev.h	3.1	92/04/19	*/
+/*	SCCS Id: @(#)sp_lev.h	3.2	96/05/08	*/
 /* Copyright (c) 1989 by Jean-Christophe Collet			  */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -30,8 +30,13 @@
  * Structures manipulated by the special levels loader & compiler
  */
 
+typedef union str_or_len {
+	char *str;
+	int   len;
+} Str_or_Len;
+
 typedef struct {
-	boolean	init_present;
+	boolean	init_present, padding;
 	char	fg, bg;
 	boolean	smoothed, joined;
 	xchar   lit, walled;
@@ -46,29 +51,25 @@ typedef struct {
 } room_door;
 
 typedef struct {
-	xchar x, y, type;
+	xchar x, y, chance, type;
 } trap;
 
 typedef struct {
-	xchar x, y, class, appear;
-	schar peaceful, asleep;
-	aligntyp	align;
+	Str_or_Len name, appear_as;
 	short id;
-	char  *name;
-	char  *appear_as;
+	aligntyp align;
+	xchar x, y, chance, class, appear;
+	schar peaceful, asleep;
 } monster;
 
 typedef struct {
-	xchar x, y, class;
-	xchar curse_state;
-	short id;
-	short spe;
+	Str_or_Len name;
 	int   corpsenm;
-	char  *name;
+	short id, spe;
+	xchar x, y, chance, class, containment;
+	schar curse_state;
 } object;
 
-#include "align.h"
-
 typedef struct {
 	xchar		x, y;
 	aligntyp	align;
@@ -105,8 +106,8 @@ typedef struct {
 	struct { xchar x1, y1, x2, y2; } inarea;
 	struct { xchar x1, y1, x2, y2; } delarea;
 	boolean in_islev, del_islev;
-	xchar rtype;
-	char *rname;
+	xchar rtype, padding;
+	Str_or_Len rname;
 } lev_region;
 
 typedef struct {
@@ -116,23 +117,20 @@ typedef struct {
 
 typedef struct {
 	xchar x, y;
-    union {
-	int length;
-	char  *text;
-    } e;
+	Str_or_Len engr;
 	xchar etype;
 } engraving;
 
 typedef struct {
-	xchar x,y;
+	xchar x, y;
 } fountain;
 
 typedef struct {
-	xchar x,y;
+	xchar x, y;
 } sink;
 
 typedef struct {
-	xchar x,y;
+	xchar x, y;
 } pool;
 
 typedef struct {
@@ -183,7 +181,7 @@ typedef struct {
 typedef struct {
 	long flags;
 	lev_init init_lev;
-	short filling;
+	schar filling;
 	char numpart;
 	mazepart **parts;
 } specialmaze;
@@ -230,6 +228,7 @@ typedef struct {
 	} src, dest;
 } corridor;
 
+/* used only by lev_comp */
 typedef struct {
 	long flags;
 	lev_init init_lev;
