@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)youprop.h	3.1	92/11/01	*/
+/*	SCCS Id: @(#)youprop.h	3.2	96/03/28	*/
 /* Copyright (c) 1989 Mike Threepoint				  */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -18,11 +18,7 @@
 #include "pm.h"
 #endif
 
-#ifdef POLYSELF
-# define UPROP(base,poly) (base || (poly))
-#else
-# define UPROP(base,poly) (base)
-#endif
+#define maybe_polyd(if_so,if_not)	(Upolyd ? (if_so) : (if_not))
 
 /* three pseudo-properties */
 #define Blindfolded	(ublindf)
@@ -33,50 +29,49 @@
    states, like swimming, wading, etc. */
 
 #define HFire_resistance	u.uprops[FIRE_RES].p_flgs
-#define Fire_resistance		UPROP(HFire_resistance,resists_fire(uasmon))
+#define Fire_resistance		(HFire_resistance || resists_fire(&youmonst))
 
 #define HCold_resistance	u.uprops[COLD_RES].p_flgs
-#define Cold_resistance		UPROP(HCold_resistance,resists_cold(uasmon))
+#define Cold_resistance		(HCold_resistance || resists_cold(&youmonst))
 
 #define HSleep_resistance	u.uprops[SLEEP_RES].p_flgs
-#define Sleep_resistance	UPROP(HSleep_resistance,resists_sleep(uasmon))
+#define Sleep_resistance	(HSleep_resistance || resists_sleep(&youmonst))
 
 #define HDisint_resistance	u.uprops[DISINT_RES].p_flgs
-#define Disint_resistance	UPROP(HDisint_resistance,resists_disint(uasmon))
+#define Disint_resistance	(HDisint_resistance || resists_disint(&youmonst))
 
 #define HShock_resistance	u.uprops[SHOCK_RES].p_flgs
-#define Shock_resistance	UPROP(HShock_resistance,resists_elec(uasmon))
+#define Shock_resistance	(HShock_resistance || resists_elec(&youmonst))
 
 #define HPoison_resistance	u.uprops[POISON_RES].p_flgs
-#define Poison_resistance	UPROP(HPoison_resistance,resists_poison(uasmon))
+#define Poison_resistance	(HPoison_resistance || resists_poison(&youmonst))
 
 #define Adornment		u.uprops[ADORNED].p_flgs
 
 #define HRegeneration		u.uprops[REGENERATION].p_flgs
-#define Regeneration		UPROP(HRegeneration,regenerates(uasmon))
+#define Regeneration		(HRegeneration || regenerates(uasmon))
 
 #define Searching		u.uprops[SEARCHING].p_flgs
 
 #define HSee_invisible		u.uprops[SEE_INVIS].p_flgs
-#define See_invisible		UPROP(HSee_invisible,perceives(uasmon))
+#define See_invisible		(HSee_invisible || perceives(uasmon))
 
 #define HInvis			u.uprops[INVIS].p_flgs
-#define Invis			UPROP(HInvis,u.usym == S_STALKER)
+#define Invis			((HInvis || pm_invisible(uasmon)) && \
+				 !(HInvis & I_BLOCKED))
 #define Invisible		(Invis && !See_invisible)
 
 #define HTeleportation		u.uprops[TELEPORT].p_flgs
-#define Teleportation		UPROP(HTeleportation,can_teleport(uasmon))
+#define Teleportation		(HTeleportation || can_teleport(uasmon))
 
 #define HTeleport_control	u.uprops[TELEPORT_CONTROL].p_flgs
-#define Teleport_control	UPROP(HTeleport_control,control_teleport(uasmon))
+#define Teleport_control	(HTeleport_control || control_teleport(uasmon))
 
-#ifdef POLYSELF
 #define Polymorph		u.uprops[POLYMORPH].p_flgs
 #define Polymorph_control	u.uprops[POLYMORPH_CONTROL].p_flgs
-#endif
 
 #define HLevitation		u.uprops[LEVITATION].p_flgs
-#define Levitation		UPROP(HLevitation,is_floater(uasmon))
+#define Levitation		(HLevitation || is_floater(uasmon))
 
 #define Stealth			u.uprops[STEALTH].p_flgs
 #define Aggravate_monster	u.uprops[AGGRAVATE_MONSTER].p_flgs
@@ -87,19 +82,19 @@
 #define Warning			u.uprops[WARNING].p_flgs
 
 #define HTelepat		u.uprops[TELEPAT].p_flgs
-#define Telepat			UPROP(HTelepat,telepathic(uasmon))
+#define Telepat			(HTelepat || telepathic(uasmon))
 
 #define Fast			u.uprops[FAST].p_flgs
 
 #define HStun			u.uprops[STUNNED].p_flgs
-#define Stunned			UPROP(HStun,u.usym==S_BAT || u.usym==S_STALKER)
+#define Stunned			(HStun || u.usym==S_BAT || u.usym==S_STALKER)
 
 #define HConfusion		u.uprops[CONFUSION].p_flgs
 #define Confusion		HConfusion
 
 #define Sick			u.uprops[SICK].p_flgs
 #define Blinded			u.uprops[BLINDED].p_flgs
-#define Blind			UPROP(Blinded || Blindfolded,!haseyes(uasmon))
+#define Blind			(Blinded || Blindfolded || !haseyes(uasmon))
 #define Sleeping		u.uprops[SLEEPING].p_flgs
 #define Wounded_legs		u.uprops[WOUNDED_LEGS].p_flgs
 #define Stoned			u.uprops[STONED].p_flgs
@@ -116,16 +111,17 @@
 #define Glib			u.uprops[GLIB].p_flgs
 #define Reflecting		u.uprops[REFLECTING].p_flgs
 #define Lifesaved		u.uprops[LIFESAVED].p_flgs
-#define Antimagic		u.uprops[ANTIMAGIC].p_flgs
+#define HAntimagic		u.uprops[ANTIMAGIC].p_flgs
+#define Antimagic		(HAntimagic || \
+				 (Upolyd && resists_magm(&youmonst)))
 #define Displaced		u.uprops[DISPLACED].p_flgs
-#define Clairvoyant		u.uprops[CLAIRVOYANT].p_flgs
+#define HClairvoyant		u.uprops[CLAIRVOYANT].p_flgs
+#define Clairvoyant		(HClairvoyant && !(HClairvoyant & I_BLOCKED))
 #define Vomiting		u.uprops[VOMITING].p_flgs
 #define Energy_regeneration	u.uprops[ENERGY_REGENERATION].p_flgs
 #define HMagical_breathing	u.uprops[MAGICAL_BREATHING].p_flgs
-#define Amphibious		UPROP(HMagical_breathing,\
-				      amphibious(uasmon))
-#define Breathless		UPROP(HMagical_breathing,\
-				      breathless(uasmon))
+#define Amphibious		(HMagical_breathing || amphibious(uasmon))
+#define Breathless		(HMagical_breathing || breathless(uasmon))
 #define Half_spell_damage	u.uprops[HALF_SPDAM].p_flgs
 #define Half_physical_damage	u.uprops[HALF_PHDAM].p_flgs
 
