@@ -1,18 +1,13 @@
-/*	SCCS Id: @(#)mthrowu.c	3.1	93/02/14	*/
+/*	SCCS Id: @(#)mthrowu.c	3.2	96/05/01	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
-#include	"hack.h"
+#include "hack.h"
 
-STATIC_DCL void FDECL(drop_throw,(struct obj *,BOOLEAN_P,int,int));
-#ifndef MUSE
-STATIC_DCL void FDECL(m_throw,(struct monst *,int,int,int,int,int,struct obj *));
-#endif
+STATIC_DCL int FDECL(drop_throw,(struct obj *,BOOLEAN_P,int,int));
 
 #define URETREATING(x,y) (distmin(u.ux,u.uy,x,y) > distmin(u.ux0,u.uy0,x,y))
 
-boolean FDECL(lined_up, (struct monst *));
-
 #ifndef OVLB
 
 STATIC_DCL const char *breathwep[];
@@ -27,7 +22,7 @@ STATIC_OVL NEARDATA const char *breathwep[] = {
 				"fire",
 				"frost",
 				"sleep gas",
-				"death",
+				"a disintegration blast",
 				"lightning",
 				"poison gas",
 				"acid",
@@ -41,7 +36,8 @@ thitu(tlev, dam, obj, name)	/* u is hit by sth, but not a monster */
 	struct obj *obj;
 	register const char *name;
 {
-	const char *onm = (obj && obj_is_pname(obj)) ? the(name) : an(name);
+	const char *onm = (obj && obj_is_pname(obj)) ? the(name) :
+		(obj->quan > 1) ? name : an(name);
 	boolean is_acid = (obj && obj->otyp == ACID_VENOM);
 
 	if(u.uac + tlev <= rnd(20)) {
@@ -51,44 +47,44 @@ thitu(tlev, dam, obj, name)	/* u is hit by sth, but not a monster */
 	} else {
 		if(Blind || !flags.verbose) You("are hit!");
 		else You("are hit by %s!", onm);
-#ifdef POLYSELF
+
 		if (obj && objects[obj->otyp].oc_material == SILVER
 				&& hates_silver(uasmon)) {
 			dam += rnd(20);
-			pline("The silver sears your flesh!");
+			pline_The("silver sears your flesh!");
 			exercise(A_CON, FALSE);
 		}
-		if (is_acid && resists_acid(uasmon))
+		if (is_acid && resists_acid(&youmonst))
 			pline("It doesn't seem to hurt you.");
 		else {
-#endif
 			if (is_acid) pline("It burns!");
 			if (Half_physical_damage) dam = (dam+1) / 2;
 			losehp(dam, name, (obj && obj_is_pname(obj)) ?
 			       KILLED_BY : KILLED_BY_AN);
 			exercise(A_STR, FALSE);
-#ifdef POLYSELF
 		}
-#endif
 		return(1);
 	}
 }
 
 /* Be sure this corresponds with what happens to player-thrown objects in
  * dothrow.c (for consistency). --KAA
+ * Returns 0 if object still exists (not destroyed).
  */
 
-STATIC_OVL void
+STATIC_OVL int
 drop_throw(obj, ohit, x, y)
 register struct obj *obj;
 boolean ohit;
 int x,y;
 {
+	int retvalu = 1;
 	int create;
 	struct monst *mtmp;
 	struct trap *t;
 
-	if (obj->otyp == CREAM_PIE || obj->oclass == VENOM_CLASS)
+	if (obj->otyp == CREAM_PIE || obj->oclass == VENOM_CLASS ||
+		    (ohit && obj->otyp == EGG))
 		create = 0;
 	else if (ohit &&
 		 ((obj->otyp >= ARROW && obj->otyp <= SHURIKEN) ||
@@ -96,24 +92,129 @@ int x,y;
 		create = !rn2(3);
 	else create = 1;
 
-	if (create && !((mtmp = m_at(x, y)) && (mtmp->mtrapped) && 
-			(t = t_at(x, y)) && ((t->ttyp == PIT) || 
+	if (create && !((mtmp = m_at(x, y)) && (mtmp->mtrapped) &&
+			(t = t_at(x, y)) && ((t->ttyp == PIT) ||
 			(t->ttyp == SPIKED_PIT)))) {
-	    if (!flooreffects(obj,x,y,"fall")) { /* don't double-dip on damage */
-		place_object(obj, x, y);
-		obj->nobj = fobj;
-		fobj = obj;
-		stackobj(fobj);
-	    }
+		int objgone = 0;
+
+		if (down_gate(x, y) != -1)
+			objgone = ship_object(obj, x, y, FALSE);
+		if (!objgone) {
+			if (!flooreffects(obj,x,y,"fall")) { /* don't double-dip on damage */
+			    place_object(obj, x, y);
+			    stackobj(obj);
+			    retvalu = 0;
+			}
+		}
 	} else obfree(obj, (struct obj*) 0);
+	return retvalu;
 }
 
 #endif /* OVLB */
 #ifdef OVL1
 
-#ifndef MUSE
-STATIC_OVL
-#endif
+/* an object launched by someone/thing other than player attacks a monster;
+   return 1 if the object has stopped moving (hit or its range used up) */
+int
+ohitmon(mtmp, otmp, range, verbose)
+struct monst *mtmp;	/* accidental target */
+struct obj *otmp;	/* missile; might be destroyed by drop_throw */
+int range;		/* how much farther will object travel if it misses */
+			/* Use -1 to signify to keep going even after hit, */
+			/* unless its gone (used for rolling_boulder_traps) */
+boolean verbose;  /* give message(s) even when you can't see what happened */
+{
+	int damage, tmp;
+	boolean vis, ismimic;
+	int objgone = 1;
+
+	ismimic = mtmp->m_ap_type && mtmp->m_ap_type != M_AP_MONSTER;
+	vis = cansee(bhitpos.x, bhitpos.y);
+
+	tmp = 5 + find_mac(mtmp) + omon_adj(mtmp, otmp, FALSE);
+	if (tmp < rnd(20)) {
+	    if (!ismimic) {
+		if (vis) miss(distant_name(otmp, xname), mtmp);
+		else if (verbose) pline("It is missed.");
+	    }
+	    if (!range) { /* Last position; object drops */
+		(void) drop_throw(otmp, 0, mtmp->mx, mtmp->my);
+		return 1;
+	    }
+	} else if (otmp->oclass == POTION_CLASS) {
+	    if (ismimic) seemimic(mtmp);
+	    mtmp->msleep = 0;
+	    if (vis) otmp->dknown = 1;
+	    potionhit(mtmp, otmp);
+	    return 1;
+	} else {
+	    damage = dmgval(otmp, mtmp);
+	    if (otmp->otyp == ACID_VENOM && resists_acid(mtmp))
+		damage = 0;
+	    if (ismimic) seemimic(mtmp);
+	    mtmp->msleep = 0;
+	    if (vis) hit(distant_name(otmp,xname), mtmp, exclam(damage));
+	    else if (verbose) pline("It is hit%s", exclam(damage));
+
+	    if (otmp->opoisoned) {
+		if (resists_poison(mtmp)) {
+		    if (vis) pline_The("poison doesn't seem to affect %s.",
+				   mon_nam(mtmp));
+		} else {
+		    if (rn2(30)) {
+			damage += rnd(6);
+		    } else {
+			if (vis) pline_The("poison was deadly...");
+			damage = mtmp->mhp;
+		    }
+		}
+	    }
+	    if (objects[otmp->otyp].oc_material == SILVER &&
+		    hates_silver(mtmp->data)) {
+		if (vis) pline_The("silver sears %s flesh!",
+				s_suffix(mon_nam(mtmp)));
+		else if (verbose) pline("Its flesh is seared!");
+	    }
+	    if (otmp->otyp == ACID_VENOM && cansee(mtmp->mx,mtmp->my)) {
+		if (resists_acid(mtmp)) {
+		    if (vis || verbose)
+			pline("%s is unaffected.", Monnam(mtmp));
+		    damage = 0;
+		} else {
+		    if (vis) pline_The("acid burns %s!", mon_nam(mtmp));
+		    else if (verbose) pline("It is burned!");
+		}
+	    }
+	    mtmp->mhp -= damage;
+	    if (mtmp->mhp < 1) {
+		if (vis || verbose)
+		    pline("%s is %s!", Monnam(mtmp),
+			(nonliving(mtmp->data) || !vis)
+			? "destroyed" : "killed");
+		mondied(mtmp);
+	    }
+
+	    if ((otmp->otyp == CREAM_PIE || otmp->otyp == BLINDING_VENOM) &&
+		   haseyes(mtmp->data)) {
+		/* note: resists_blnd() doesn't apply here */
+		if (vis) pline("%s is blinded by %s.",
+				Monnam(mtmp), the(xname(otmp)));
+		mtmp->mcansee = 0;
+		tmp = (int)mtmp->mblinded + rnd(25) + 20;
+		if (tmp > 127) tmp = 127;
+		mtmp->mblinded = tmp;
+	    }
+
+	    objgone = drop_throw(otmp, 1, bhitpos.x, bhitpos.y);
+	    if (!objgone && range == -1) {  /* special case */
+		    obj_extract_self(otmp); /* free it for motion again */
+		    return 0;
+	    }
+	    return 1;
+	}
+	return 0;
+}
+
 void
 m_throw(mon, x, y, dx, dy, range, obj)
 	register struct monst *mon;
@@ -123,42 +224,37 @@ m_throw(mon, x, y, dx, dy, range, obj)
 	register struct monst *mtmp;
 	struct obj *singleobj;
 	char sym = obj->oclass;
-	int damage;
-	int hitu, blindinc=0;
+	int hitu, blindinc = 0;
 
 	bhitpos.x = x;
 	bhitpos.y = y;
 
-	singleobj = splitobj(obj, obj->quan - 1L);
-	/* splitobj leaves the new object in the chain (i.e. the monster's
-	 * inventory).  Remove it.  We can do this in 1 line, but it's highly
-	 * dependent on the fact that we know splitobj() places it immediately
-	 * after obj.
-	 */
-	obj->nobj = singleobj->nobj;
-	/* Get rid of object.  This cannot be done later on; what if the
-	 * player dies before then, leaving the monster with 0 daggers?
-	 * (This caused the infamous 2^32-1 orcish dagger bug).
-	 */
-	if (!obj->quan) {
-	    if(obj->oclass == VENOM_CLASS) {
-		/* venom is not in the monster's inventory chain */
-		dealloc_obj(obj);
-	    } else {
-#ifdef MUSE
-		/* not possibly_unwield, which checks the object's */
-		/* location, not its existence */
-		if (MON_WEP(mon) == obj) {
-			obj->owornmask &= ~W_WEP;
-			MON_NOWEP(mon);
-		}
-#endif
-		m_useup(mon, obj);
+	if (obj->quan == 1L) {
+	    /*
+	     * Remove object from minvent.  This cannot be done later on;
+	     * what if the player dies before then, leaving the monster
+	     * with 0 daggers?  (This caused the infamous 2^32-1 orcish
+	     * dagger bug).
+	     *
+	     * VENOM is not in minvent - it should already be OBJ_FREE.
+	     * The extract below does nothing.
+	     */
+
+	    /* not possibly_unwield, which checks the object's */
+	    /* location, not its existence */
+	    if (MON_WEP(mon) == obj) {
+		    obj->owornmask &= ~W_WEP;
+		    MON_NOWEP(mon);
 	    }
+	    obj_extract_self(obj);
+	    singleobj = obj;
+	    obj = (struct obj *) 0;
+	} else {
+	    singleobj = splitobj(obj, obj->quan - 1L);
+	    obj_extract_self(singleobj);
 	}
-#ifdef MUSE
+
 	singleobj->owornmask = 0; /* threw one of multiple weapons in hand? */
-#endif
 
 	if (singleobj->cursed && (dx || dy) && !rn2(7)) {
 	    if(canseemon(mon) && flags.verbose) {
@@ -176,10 +272,8 @@ m_throw(mon, x, y, dx, dy, range, obj)
 	    if((!dx && !dy)
 	       || !isok(bhitpos.x+dx,bhitpos.y+dy)
 	       /* missile hits the wall */
-	       || IS_WALL(levl[bhitpos.x+dx][bhitpos.y+dy].typ)
-	       || levl[bhitpos.x+dx][bhitpos.y+dy].typ == SDOOR
-	       || levl[bhitpos.x+dx][bhitpos.y+dy].typ == SCORR) {
-		drop_throw(singleobj, 0, bhitpos.x, bhitpos.y);
+	       || IS_ROCK(levl[bhitpos.x+dx][bhitpos.y+dy].typ)) {
+		(void) drop_throw(singleobj, 0, bhitpos.x, bhitpos.y);
 		return;
 	    }
 	}
@@ -190,111 +284,19 @@ m_throw(mon, x, y, dx, dy, range, obj)
 	 */
 	if (sym) tmp_at(DISP_FLASH, obj_to_glyph(singleobj));
 	while(range-- > 0) { /* Actually the loop is always exited by break */
-		boolean vis;
-
 		bhitpos.x += dx;
 		bhitpos.y += dy;
-		vis = cansee(bhitpos.x, bhitpos.y);
-		if(MON_AT(bhitpos.x, bhitpos.y)) {
-		    boolean ismimic;
-
-		    mtmp = m_at(bhitpos.x,bhitpos.y);
-		    ismimic = mtmp->m_ap_type &&
-			mtmp->m_ap_type != M_AP_MONSTER;
-
-		    /* don't use distance/size modifiers since target was u */
-		    if(find_mac(mtmp) + 8 + singleobj->spe <= rnd(20)) {
-			if (!ismimic) {
-			    if (!vis) pline("It is missed.");
-			    else miss(distant_name(singleobj,xname), mtmp);
-			}
-			if (!range) { /* Last position; object drops */
-			    drop_throw(singleobj, 0, mtmp->mx, mtmp->my);
-			    break;
-			}
-#ifdef MUSE
-		    } else if (singleobj->oclass == POTION_CLASS) {
-			if (ismimic) seemimic(mtmp);
-			if (vis) singleobj->dknown = 1;
-			potionhit(mtmp, singleobj);
+		if ((mtmp = m_at(bhitpos.x, bhitpos.y)) != 0) {
+		    if (ohitmon(mtmp, singleobj, range, TRUE))
 			break;
-#endif
-		    } else {
-			damage = dmgval(singleobj, mtmp->data);
-			if (damage < 1) damage = 1;
-			if (singleobj->otyp==ACID_VENOM && resists_acid(mtmp->data))
-			    damage = 0;
-			if (ismimic) seemimic(mtmp);
-			if (!vis) pline("It is hit%s", exclam(damage));
-			else hit(distant_name(singleobj,xname),
-							mtmp,exclam(damage));
-			if (singleobj->opoisoned) {
-			    if (resists_poison(mtmp->data)) {
-				if (vis)
-				  pline("The poison doesn't seem to affect %s.",
-								mon_nam(mtmp));
-			    } else {
-				if (rn2(30)) damage += rnd(6);
-				else {
-				    if (vis)
-					pline("The poison was deadly...");
-				    damage = mtmp->mhp;
-				}
-			    }
-			}
-			if (objects[singleobj->otyp].oc_material == SILVER
-				&& hates_silver(mtmp->data)) {
-			    if (vis) pline("The silver sears %s's flesh!",
-				mon_nam(mtmp));
-			    else pline("Its flesh is seared!");
-			}
-			if (singleobj->otyp==ACID_VENOM && cansee(mtmp->mx,mtmp->my)){
-			    if (resists_acid(mtmp->data)) {
-				pline("%s is unaffected.", vis ? Monnam(mtmp)
-					: "It");
-				damage = 0;
-			    } else if (vis)
-				pline("The acid burns %s!", mon_nam(mtmp));
-			    else pline("It is burned!");
-			}
-			mtmp->mhp -= damage;
-			if(mtmp->mhp < 1) {
-			    pline("%s is %s!", vis ? Monnam(mtmp) : "It",
-			       (is_demon(mtmp->data) || 
-					is_undead(mtmp->data) || !vis) ?
-				 "destroyed" : "killed");
-			    mondied(mtmp);
-			}
-
-			if(((singleobj->otyp == CREAM_PIE) ||
-			    (singleobj->otyp == BLINDING_VENOM))
-			   && haseyes(mtmp->data)) {
-			    if (vis)
-				pline("%s is blinded by %s.",
-				      Monnam(mtmp), the(xname(singleobj)));
-			    if(mtmp->msleep) mtmp->msleep = 0;
-			    mtmp->mcansee = 0;
-			    {
-				register unsigned rnd_tmp = rnd(25) + 20;
-				if((mtmp->mblinded + rnd_tmp) > 127)
-					mtmp->mblinded = 127;
-				else mtmp->mblinded += rnd_tmp;
-			    }
-			}
-			drop_throw(singleobj, 1, bhitpos.x, bhitpos.y);
-			break;
-		    }
-		}
-		if (bhitpos.x == u.ux && bhitpos.y == u.uy) {
+		} else if (bhitpos.x == u.ux && bhitpos.y == u.uy) {
 			if (multi) nomul(0);
 
-#ifdef MUSE
 			if (singleobj->oclass == POTION_CLASS) {
 			    if (!Blind) singleobj->dknown = 1;
 			    potionhit(&youmonst, singleobj);
 			    break;
 			}
-#endif
 			switch(singleobj->otyp) {
 			    int dam, hitv;
 			    case CREAM_PIE:
@@ -302,64 +304,60 @@ m_throw(mon, x, y, dx, dy, range, obj)
 				hitu = thitu(8, 0, singleobj, xname(singleobj));
 				break;
 			    default:
-				dam = dmgval(singleobj, uasmon);
+				dam = dmgval(singleobj, &youmonst);
 				hitv = 3 - distmin(u.ux,u.uy, mon->mx,mon->my);
 				if (hitv < -4) hitv = -4;
 				if (is_elf(mon->data) &&
 				    objects[singleobj->otyp].w_propellor
 								== WP_BOW) {
 				    hitv++;
-#ifdef MUSE
 				    if (MON_WEP(mon) &&
 					MON_WEP(mon)->otyp == ELVEN_BOW)
 					hitv++;
-#endif
 				    if(singleobj->otyp == ELVEN_ARROW) dam++;
 				}
-#ifdef POLYSELF
 				if (bigmonst(uasmon)) hitv++;
-#endif
 				hitv += 8+singleobj->spe;
 
 				if (dam < 1) dam = 1;
 				hitu = thitu(hitv, dam,
 					singleobj, xname(singleobj));
 			}
-			if (hitu && singleobj->opoisoned)
-			    /* it's safe to call xname twice because it's the
-			       same object both times... */
-			    poisoned(xname(singleobj), A_STR, xname(singleobj), 10);
+			if (hitu && singleobj->opoisoned) {
+			    char *singlename = xname(singleobj);
+			    poisoned(singlename, A_STR, singlename, 10);
+			}
 			if(hitu && (singleobj->otyp == CREAM_PIE ||
 				     singleobj->otyp == BLINDING_VENOM)) {
 			    blindinc = rnd(25);
 			    if(singleobj->otyp == CREAM_PIE) {
 				if(!Blind) pline("Yecch!  You've been creamed.");
-				else	pline("There's something sticky all over your %s.", body_part(FACE));
+				else	pline("There's %s sticky all over your %s.",
+						something,
+						body_part(FACE));
 			    } else {	/* venom in the eyes */
 				if(Blindfolded) /* nothing */ ;
-				else if(!Blind) pline("The venom blinds you.");
+				else if(!Blind) pline_The("venom blinds you.");
 				else	Your("%s sting.",
 					makeplural(body_part(EYE)));
 			    }
 			}
 			stop_occupation();
 			if (hitu || !range) {
-			    drop_throw(singleobj, hitu, u.ux, u.uy);
+			    (void) drop_throw(singleobj, hitu, u.ux, u.uy);
 			    break;
 			}
 		} else if (!range	/* reached end of path */
 			/* missile hits edge of screen */
 			|| !isok(bhitpos.x+dx,bhitpos.y+dy)
 			/* missile hits the wall */
-			|| IS_WALL(levl[bhitpos.x+dx][bhitpos.y+dy].typ)
-			|| levl[bhitpos.x+dx][bhitpos.y+dy].typ == SDOOR
-			|| levl[bhitpos.x+dx][bhitpos.y+dy].typ == SCORR
+			|| IS_ROCK(levl[bhitpos.x+dx][bhitpos.y+dy].typ)
 #ifdef SINKS
 			/* Thrown objects "sink" */
 			|| IS_SINK(levl[bhitpos.x][bhitpos.y].typ)
 #endif
 								) {
-		    drop_throw(singleobj, 0, bhitpos.x, bhitpos.y);
+		    (void) drop_throw(singleobj, 0, bhitpos.x, bhitpos.y);
 		    break;
 		}
 		tmp_at(bhitpos.x, bhitpos.y);
@@ -378,34 +376,23 @@ m_throw(mon, x, y, dx, dy, range, obj)
 #endif /* OVL1 */
 #ifdef OVLB
 
-/* Remove an item from the monster's inventory.
- */
+/* Remove an item from the monster's inventory and destroy it. */
 void
 m_useup(mon, obj)
 struct monst *mon;
 struct obj *obj;
 {
-	struct obj *otmp, *prev;
-
 	if (obj->quan > 1L) {
 		obj->quan--;
-		return;
-	}
-	prev = ((struct obj *) 0);
-	for (otmp = mon->minvent; otmp; otmp = otmp->nobj) {
-		if (otmp == obj) {
-			if (prev)
-				prev->nobj = obj->nobj;
-			else
-				mon->minvent = obj->nobj;
-			dealloc_obj(obj);
-			break;
+	} else {
+		obj_extract_self(obj);
+		possibly_unwield(mon);
+		if (obj->owornmask) {
+		    mon->misc_worn_check &= ~obj->owornmask;
+		    update_mon_intrinsics(mon, obj, FALSE);
 		}
-		prev = otmp;
+		dealloc_obj(obj);
 	}
-#ifdef MUSE
-	possibly_unwield(mon);
-#endif
 }
 
 #endif /* OVLB */
@@ -419,14 +406,13 @@ register struct monst *mtmp;
 	register xchar x, y;
 
 	if(lined_up(mtmp)) {
-#ifdef MUSE
 	    if (mtmp->weapon_check == NEED_WEAPON || !MON_WEP(mtmp)) {
 		mtmp->weapon_check = NEED_RANGED_WEAPON;
 		/* mon_wield_item resets weapon_check as appropriate */
 		if(mon_wield_item(mtmp) != 0) return;
 	    }
-#endif
-	    if((otmp = select_rwep(mtmp))) {
+	    otmp = select_rwep(mtmp);
+	    if (otmp) {
 		/* If you are coming toward the monster, the monster
 		 * should try to soften you up with missiles.  If you are
 		 * going away, you are probably hurt or running.  Give
@@ -449,7 +435,7 @@ register struct monst *mtmp;
 			      the(singular(otmp, xname)) :
 			      an(singular(otmp, xname)));
 		    }
-		    m_throw(mtmp, mtmp->mx, mtmp->my, sgn(tbx), sgn(tby), 
+		    m_throw(mtmp, mtmp->mx, mtmp->my, sgn(tbx), sgn(tby),
 			distmin(mtmp->mx,mtmp->my,mtmp->mux,mtmp->muy), otmp);
 		    nomul(0);
 		    return;
@@ -471,7 +457,7 @@ register struct attack *mattk;
 	if(mtmp->mcan) {
 
 	    if(flags.soundok)
-		pline("A dry rattle comes from %s throat", 
+		pline("A dry rattle comes from %s throat",
 		                      s_suffix(mon_nam(mtmp)));
 	    return 0;
 	}
@@ -491,7 +477,7 @@ register struct attack *mattk;
 		if(!rn2(BOLT_LIM-distmin(mtmp->mx,mtmp->my,mtmp->mux,mtmp->muy))) {
 		    if (canseemon(mtmp))
 			pline("%s spits venom!", Monnam(mtmp));
-		    m_throw(mtmp, mtmp->mx, mtmp->my, sgn(tbx), sgn(tby), 
+		    m_throw(mtmp, mtmp->mx, mtmp->my, sgn(tbx), sgn(tby),
 			distmin(mtmp->mx,mtmp->my,mtmp->mux,mtmp->muy), otmp);
 		    nomul(0);
 		    return 0;
@@ -518,7 +504,7 @@ breamu(mtmp, mattk)			/* monster breathes at you (ranged) */
 		    if(canseemon(mtmp))
 			pline("%s coughs.", Monnam(mtmp));
 		    else
-			You("hear a cough.");
+			You_hear("a cough.");
 		}
 		return(0);
 	    }
@@ -552,9 +538,12 @@ register xchar ax, ay, bx, by;
 	tbx = ax - bx;	/* These two values are set for use */
 	tby = ay - by;	/* after successful return.	    */
 
+	/* sometimes displacement makes a monster think that you're at its
+	   own location; prevent it from throwing and zapping in that case */
+	if (!tbx && !tby) return FALSE;
+
 	if((!tbx || !tby || abs(tbx) == abs(tby)) /* straight line or diagonal */
 	   && distmin(tbx, tby, 0, 0) < BOLT_LIM) {
-
 	    if(ax == u.ux && ay == u.uy) return((boolean)(couldsee(bx,by)));
 	    else if(clear_path(ax,ay,bx,by)) return TRUE;
 	}
