@@ -1,37 +1,45 @@
-/*	SCCS Id: @(#)o_init.c	3.1	93/05/25	*/
+/*	SCCS Id: @(#)o_init.c	3.2	94/07/10	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
-#include	"hack.h"		/* for typedefs */
+#include "hack.h"
+#include "lev.h"	/* save & restore info */
 
 static void NDECL(setgemprobs);
 static void FDECL(shuffle,(int,int,BOOLEAN_P));
+static void NDECL(shuffle_all);
 static boolean FDECL(interesting_to_discover,(int));
 
-/* note that NROFOBJECTS is the number of legal objects, which does not count
- * the strange object and null object that take up positions 0 and NROFOBJECTS+1
- * in the objects array
+
+static NEARDATA short disco[NUM_OBJECTS] = DUMMY;
+
+#ifdef USE_TILES
+static void NDECL(shuffle_tiles);
+extern short glyph2tile[];	/* from tile.c */
+
+/* Shuffle tile assignments to match descriptions, so a red potion isn't
+ * displayed with a blue tile and so on.
+ *
+ * Tile assignments are not saved, and shouldn't be so that a game can
+ * be resumed on an otherwise identical non-tile-using binary, so we have
+ * to reshuffle the assignments from oc_descr_idx information when a game
+ * is restored.  So might as well do that the first time instead of writing
+ * another routine.
  */
-#define TOTAL_OBJS	(NROFOBJECTS+2)
-
-const char obj_symbols[] = {
-	ILLOBJ_CLASS, AMULET_CLASS, GOLD_CLASS, FOOD_CLASS, WEAPON_CLASS,
-	TOOL_CLASS, BALL_CLASS, CHAIN_CLASS, ROCK_CLASS, ARMOR_CLASS,
-	POTION_CLASS, SCROLL_CLASS, WAND_CLASS,
-	SPBOOK_CLASS, RING_CLASS, GEM_CLASS, 0 };
-
-static NEARDATA short disco[TOTAL_OBJS] = DUMMY;
-
-int
-letindex(acls)
-register char acls;
+static void
+shuffle_tiles()
 {
-register int i = 0;
-register char ch;
-	while ((ch = obj_symbols[i++]) != 0)
-		if (ch == acls) return(i);
-	return(0);
+	int i;
+	short tmp_tilemap[NUM_OBJECTS];
+
+	for (i = 0; i < NUM_OBJECTS; i++)
+		tmp_tilemap[i] =
+			glyph2tile[objects[i].oc_descr_idx + GLYPH_OBJ_OFF];
+
+	for (i = 0; i < NUM_OBJECTS; i++)
+		glyph2tile[i + GLYPH_OBJ_OFF] = tmp_tilemap[i];
 }
+#endif	/* USE_TILES */
 
 static void
 setgemprobs()
@@ -40,13 +48,13 @@ setgemprobs()
 	int lev = (ledger_no(&u.uz) > maxledgerno())
 				? maxledgerno() : ledger_no(&u.uz);
 
-	first = bases[letindex(GEM_CLASS)];
+	first = bases[GEM_CLASS];
 
 	for(j = 0; j < 9-lev/3; j++)
 		objects[first+j].oc_prob = 0;
 	first += j;
 	if (first > LAST_GEM || objects[first].oc_class != GEM_CLASS ||
-	    OBJ_NAME(objects[first]) == NULL) {
+	    OBJ_NAME(objects[first]) == (char *)0) {
 		raw_printf("Not enough gems? - first=%d j=%d LAST_GEM=%d",
 			first, j, LAST_GEM);
 		wait_synch();
@@ -58,20 +66,30 @@ setgemprobs()
 /* shuffle descriptions on objects o_low to o_high */
 static void
 shuffle(o_low, o_high, domaterial)
-	register int o_low, o_high;
-	register boolean domaterial;
+	int o_low, o_high;
+	boolean domaterial;
 {
-	register int i, j;
+	int i, j, num_to_shuffle;
+	short sw;
 #ifdef TEXTCOLOR
 	int color;
 #endif /* TEXTCOLOR */
-	register short sw;
+
+	for (num_to_shuffle = 0, j=o_low; j <= o_high; j++)
+		if (!objects[j].oc_name_known) num_to_shuffle++;
+	if (num_to_shuffle < 2) return;
 
 	for (j=o_low; j <= o_high; j++) {
-		i = o_low + rn2(j+1-o_low);
+		if (objects[j].oc_name_known) continue;
+		do
+			i = j + rn2(o_high-j+1);
+		while (objects[i].oc_name_known);
 		sw = objects[j].oc_descr_idx;
 		objects[j].oc_descr_idx = objects[i].oc_descr_idx;
 		objects[i].oc_descr_idx = sw;
+		sw = objects[j].oc_tough;
+		objects[j].oc_tough = objects[i].oc_tough;
+		objects[i].oc_tough = sw;
 #ifdef TEXTCOLOR
 		color = objects[j].oc_color;
 		objects[j].oc_color = objects[i].oc_color;
@@ -89,8 +107,8 @@ shuffle(o_low, o_high, domaterial)
 void
 init_objects()
 {
-register int i, j, first, last, sum, end;
-register char acls;
+register int i, first, last, sum;
+register char oclass;
 #ifdef TEXTCOLOR
 # define COPY_OBJ_DESCR(o_dst,o_src) \
 			o_dst.oc_descr_idx = o_src.oc_descr_idx,\
@@ -102,48 +120,23 @@ register char acls;
 	/* bug fix to prevent "initialization error" abort on Intel Xenix.
 	 * reported by mikew@semike
 	 */
-	for (i = 0; i < sizeof(obj_symbols); i++)
+	for (i = 0; i < MAXOCLASSES; i++)
 		bases[i] = 0;
 	/* initialize object descriptions */
-	for (i = 0; i < TOTAL_OBJS; i++)
+	for (i = 0; i < NUM_OBJECTS; i++)
 		objects[i].oc_name_idx = objects[i].oc_descr_idx = i;
-	/* moved to after u_init()
-	init_artifacts();
-	*/
 	/* init base; if probs given check that they add up to 1000,
-	   otherwise compute probs; shuffle descriptions */
-	end = TOTAL_OBJS;
+	   otherwise compute probs */
 	first = 0;
-	while( first < end ) {
-		acls = objects[first].oc_class;
+	while( first < NUM_OBJECTS ) {
+		oclass = objects[first].oc_class;
 		last = first+1;
-		while (last < end && objects[last].oc_class == acls) last++;
-		i = letindex(acls);
-		if ((!i && acls != ILLOBJ_CLASS && acls != VENOM_CLASS) ||
-								bases[i] != 0)
-			error("initialization error for object class %d", acls);
-		bases[i] = first;
+		while (last < NUM_OBJECTS && objects[last].oc_class == oclass) last++;
+		bases[(int)oclass] = first;
 
-		if (acls == GEM_CLASS) setgemprobs();
-	check:
-		sum = 0;
-		for(j = first; j < last; j++) sum += objects[j].oc_prob;
-		if(sum == 0) {
-			for(j = first; j < last; j++)
-			    objects[j].oc_prob = (1000+j-first)/(last-first);
-			goto check;
-		}
-		if(sum != 1000)
-			error("init-prob error for %d (%d%%)", acls, sum);
+		if (oclass == GEM_CLASS) {
+			setgemprobs();
 
-		if (OBJ_DESCR(objects[first]) != NULL &&
-		   acls != TOOL_CLASS && acls != WEAPON_CLASS && acls != ARMOR_CLASS) {
-
-		    /* shuffle, also some additional descriptions */
-		    while (last < end && objects[last].oc_class == acls)
-			last++;
-		    j = last;
-		    if (acls == GEM_CLASS) {
 			if (rn2(2)) { /* change turquoise from green to blue? */
 			    COPY_OBJ_DESCR(objects[TURQUOISE],objects[SAPPHIRE]);
 			}
@@ -162,23 +155,60 @@ register char acls;
 				COPY_OBJ_DESCR(objects[FLUORITE],objects[EMERALD]);
 				break;
 			}
-		    } else {
-			if (acls == POTION_CLASS)
-			    j--;  /* only water has a fixed description */
-			else if (acls == AMULET_CLASS ||
-				 acls == SCROLL_CLASS ||
-				 acls == SPBOOK_CLASS)
-			    do { j--; }
-			    while (!objects[j].oc_magic || objects[j].oc_unique);
+		}
+	check:
+		sum = 0;
+		for(i = first; i < last; i++) sum += objects[i].oc_prob;
+		if(sum == 0) {
+			for(i = first; i < last; i++)
+			    objects[i].oc_prob = (1000+i-first)/(last-first);
+			goto check;
+		}
+		if(sum != 1000)
+			error("init-prob error for class %d (%d%%)", oclass, sum);
+		first = last;
+	}
+	/* shuffle descriptions */
+	shuffle_all();
+#ifdef USE_TILES
+	shuffle_tiles();
+#endif
+}
+
+static void
+shuffle_all()
+{
+	int first, last, oclass;
+
+	for (oclass = 1; oclass < MAXOCLASSES; oclass++) {
+		first = bases[oclass];
+		last = first+1;
+		while (last < NUM_OBJECTS && objects[last].oc_class == oclass)
+			last++;
+
+		if (OBJ_DESCR(objects[first]) != (char *)0 &&
+				oclass != TOOL_CLASS &&
+				oclass != WEAPON_CLASS &&
+				oclass != ARMOR_CLASS &&
+				oclass != GEM_CLASS) {
+			int j = last-1;
+
+			if (oclass == POTION_CLASS)
+			    j -= 1;  /* only water has a fixed description */
+			else if (oclass == AMULET_CLASS ||
+				 oclass == SCROLL_CLASS ||
+				 oclass == SPBOOK_CLASS) {
+			    while (!objects[j].oc_magic || objects[j].oc_unique)
+				j--;
+			}
+
 			/* non-magical amulets, scrolls, and spellbooks
 			 * (ex. imitation Amulets, blank, scrolls of mail)
 			 * and one-of-a-kind magical artifacts at the end of
 			 * their class in objects[] have fixed descriptions.
 			 */
-			shuffle(first, --j, TRUE);
-		    }
+			shuffle(first, j, TRUE);
 		}
-		first = last;
 	}
 
 	/* shuffle the helmets */
@@ -204,8 +234,6 @@ find_skates()
     for (i = SPEED_BOOTS; i <= LEVITATION_BOOTS; i++)
 	if ((s = OBJ_DESCR(objects[i])) != 0 && !strcmp(s, "snow boots"))
 	    return i;
-    /* special case:  description of elven boots is nulled out for elf role */
-    if (pl_character[0] == 'E') return ELVEN_BOOTS;
 
     impossible("snow boots not found?");
     return -1;	/* not 0, or caller would try again each move */
@@ -218,25 +246,33 @@ oinit()			/* level dependent initialization */
 }
 
 void
-savenames(fd)
-register int fd;
+savenames(fd, mode)
+int fd, mode;
 {
 	register int i;
 	unsigned int len;
 
-	bwrite(fd, (genericptr_t)bases, MAXOCLASSES * sizeof *bases);
-	bwrite(fd, (genericptr_t)disco, sizeof disco);
-	bwrite(fd, (genericptr_t)objects, sizeof(struct objclass) * TOTAL_OBJS);
+	if (perform_bwrite(mode)) {
+	    bwrite(fd, (genericptr_t)bases, sizeof bases);
+	    bwrite(fd, (genericptr_t)disco, sizeof disco);
+	    bwrite(fd, (genericptr_t)objects,
+		   sizeof(struct objclass) * NUM_OBJECTS);
+	}
 	/* as long as we use only one version of Hack we
 	   need not save oc_name and oc_descr, but we must save
 	   oc_uname for all objects */
-	for(i=0; i < TOTAL_OBJS; i++) {
-		if(objects[i].oc_uname) {
-			len = strlen(objects[i].oc_uname)+1;
-			bwrite(fd, (genericptr_t)&len, sizeof len);
-			bwrite(fd, (genericptr_t)objects[i].oc_uname, len);
+	for (i = 0; i < NUM_OBJECTS; i++)
+	    if (objects[i].oc_uname) {
+		if (perform_bwrite(mode)) {
+		    len = strlen(objects[i].oc_uname)+1;
+		    bwrite(fd, (genericptr_t)&len, sizeof len);
+		    bwrite(fd, (genericptr_t)objects[i].oc_uname, len);
 		}
-	}
+		if (release_data(mode)) {
+		    free((genericptr_t)objects[i].oc_uname);
+		    objects[i].oc_uname = 0;
+		}
+	    }
 }
 
 void
@@ -246,16 +282,18 @@ register int fd;
 	register int i;
 	unsigned int len;
 
-	mread(fd, (genericptr_t) bases, MAXOCLASSES * sizeof *bases);
+	mread(fd, (genericptr_t) bases, sizeof bases);
 	mread(fd, (genericptr_t) disco, sizeof disco);
-	mread(fd, (genericptr_t) objects, sizeof(struct objclass) * TOTAL_OBJS);
-	for(i=0; i < TOTAL_OBJS; i++) {
-		if (objects[i].oc_uname) {
-			mread(fd, (genericptr_t) &len, sizeof len);
-			objects[i].oc_uname = (char *) alloc(len);
-			mread(fd, (genericptr_t)objects[i].oc_uname, len);
-		}
-	}
+	mread(fd, (genericptr_t) objects, sizeof(struct objclass) * NUM_OBJECTS);
+	for (i = 0; i < NUM_OBJECTS; i++)
+	    if (objects[i].oc_uname) {
+		mread(fd, (genericptr_t) &len, sizeof len);
+		objects[i].oc_uname = (char *) alloc(len);
+		mread(fd, (genericptr_t)objects[i].oc_uname, len);
+	    }
+#ifdef USE_TILES
+	shuffle_tiles();
+#endif
 }
 
 void
@@ -270,7 +308,7 @@ boolean mark_as_known;
 	   uname'd) or the next open slot; one or the other will be found
 	   before we reach the next class...
 	 */
-	for (dindx = bases[letindex(acls)]; disco[dindx] != 0; dindx++)
+	for (dindx = bases[acls]; disco[dindx] != 0; dindx++)
 	    if (disco[dindx] == oindx) break;
 	disco[dindx] = oindx;
 
@@ -291,8 +329,8 @@ register int oindx;
 	register boolean found = FALSE;
 
 	/* find the object; shift those behind it forward one slot */
-	for (dindx = bases[letindex(acls)];
-	      dindx <= NROFOBJECTS && disco[dindx] != 0
+	for (dindx = bases[acls];
+	      dindx < NUM_OBJECTS && disco[dindx] != 0
 		&& objects[dindx].oc_class == acls; dindx++)
 	    if (found)
 		disco[dindx-1] = disco[dindx];
@@ -309,8 +347,9 @@ static boolean
 interesting_to_discover(i)
 register int i;
 {
-    return((boolean)(objects[i].oc_uname != NULL ||
-		(objects[i].oc_name_known && OBJ_DESCR(objects[i]) != NULL)));
+    if (objects[i].oc_pre_discovered) return FALSE;
+    return((boolean)(objects[i].oc_uname != (char *)0 ||
+	    (objects[i].oc_name_known && OBJ_DESCR(objects[i]) != (char *)0)));
 }
 
 int
@@ -336,8 +375,8 @@ dodiscovered()				/* free after Robert Viduya */
     for (s = classes; *s; s++) {
 	oclass = *s;
 	prev_class = oclass + 1;	/* forced different from oclass */
-	for (i = bases[letindex(oclass)];
-	     i <= NROFOBJECTS && objects[i].oc_class == oclass; i++) {
+	for (i = bases[(int)oclass];
+	     i < NUM_OBJECTS && objects[i].oc_class == oclass; i++) {
 	    if ((dis = disco[i]) && interesting_to_discover(dis)) {
 		ct++;
 		if (oclass != prev_class) {
