@@ -1,22 +1,19 @@
-/*	SCCS Id: @(#)pager.c	3.1	92/09/01		  */
+/*	SCCS Id: @(#)pager.c	3.2	96/02/14	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
 /* This file contains the command routines dowhatis() and dohelp() and */
 /* a few other help related facilities */
 
-#include	"hack.h"
+#include "hack.h"
+#include "dlb.h"
 
-#ifndef SEEK_SET
-#define SEEK_SET 0
-#endif
-
-static boolean FDECL(is_swallow_sym, (UCHAR_P));
+static boolean FDECL(is_swallow_sym, (int));
 static int FDECL(append_str, (char *, const char *));
 static void FDECL(lookat, (int, int, char *));
 static void FDECL(checkfile, (char *, BOOLEAN_P));
 static int FDECL(do_look, (BOOLEAN_P));
-static char NDECL(help_menu);
+static boolean FDECL(help_menu, (int *));
 #ifdef PORT_HELP
 extern void NDECL(port_help);
 #endif
@@ -24,11 +21,11 @@ extern void NDECL(port_help);
 /* Returns "true" for characters that could represent a monster's stomach. */
 static boolean
 is_swallow_sym(c)
-    uchar c;
+int c;
 {
     int i;
     for (i = S_sw_tl; i <= S_sw_br; i++)
-	if (showsyms[i] == c) return TRUE;
+	if ((int)showsyms[i] == c) return TRUE;
     return FALSE;
 }
 
@@ -61,18 +58,15 @@ lookat(x, y, buf)
     char *buf;
 {
     register struct monst *mtmp;
-    struct trap *trap;
-    register char *s, *t;
     int glyph;
 
     buf[0] = 0;
     glyph = glyph_at(x,y);
     if (u.ux == x && u.uy == y && canseeself()) {
-	Sprintf(buf, "%s called %s",
-#ifdef POLYSELF
-		u.mtimedone ? mons[u.umonnum].mname :
-#endif
-		player_mon()->mname, plname);
+	Sprintf(buf, "%s%s called %s",
+		Invis ? "invisible " : "",
+		u.mtimedone ? mons[u.umonnum].mname : player_mon()->mname,
+		plname);
     }
     else if (u.uswallow) {
 	/* all locations when swallowed other than the hero are the monster */
@@ -87,19 +81,18 @@ lookat(x, y, buf)
 	    register boolean hp = (mtmp->data == &mons[PM_HIGH_PRIEST]);
 
 	    Sprintf(buf, "%s%s%s",
+		    (mtmp->mx != x || mtmp->my != y) ?
+			((mtmp->isshk && !Hallucination)
+				? "tail of " : "tail of a ") : "",
 		    (!hp && mtmp->mtame && !Hallucination) ? "tame " :
 		    (!hp && mtmp->mpeaceful && !Hallucination) ?
 		                                          "peaceful " : "",
-		    (hp ? "high priest" : l_monnam(mtmp)),
-		    u.ustuck == mtmp ?
-#ifdef POLYSELF
-			(u.mtimedone ? ", being held" :
-#endif
-			", holding you"
-#ifdef POLYSELF
-			)
-#endif
-			: "");
+		    (hp ? "high priest" : l_monnam(mtmp)));
+	    if (u.ustuck == mtmp)
+		Strcat(buf, (Upolyd && sticks(uasmon)) ?
+			", being held" : ", holding you");
+	    if (mtmp->mleashed)
+		Strcat(buf, ", leashed to you");
 	}
     }
     else if (glyph_is_object(glyph)) {
@@ -110,41 +103,50 @@ lookat(x, y, buf)
 		otmp = mksobj(glyph_to_obj(glyph), FALSE, FALSE);
 		if(otmp->oclass == GOLD_CLASS)
 		    otmp->quan = 2L; /* to force pluralization */
+		else if (otmp->otyp == SLIME_MOLD)
+		    otmp->spe = current_fruit;	/* give the fruit a type */
 		Strcpy(buf, distant_name(otmp, xname));
 		dealloc_obj(otmp);
 	    }
 	} else
 	    Strcpy(buf, distant_name(otmp, xname));
+
+	if (levl[x][y].typ == STONE || levl[x][y].typ == SCORR)
+	    Strcat(buf, " embedded in stone");
+	else if (IS_WALL(levl[x][y].typ) || levl[x][y].typ == SDOOR)
+	    Strcat(buf, " embedded in a wall");
+	else if (closed_door(x,y))
+	    Strcat(buf, " embedded in a door");
+	else if (is_pool(x,y))
+	    Strcat(buf, " in water");
+	else if (is_lava(x,y))
+	    Strcat(buf, " in molten lava");	/* [can this ever happen?] */
     }
     else if (glyph_is_trap(glyph)) {
-	if (trap = t_at(x, y)) {
-	    if (trap->ttyp == WEB)
-		Strcpy(buf, "web");
-	    else {
-		Strcpy(buf, traps[ Hallucination ?
-				     rn2(TRAPNUM-3)+3 : trap->ttyp]);
-		/* strip leading garbage */
-		for (s = buf; *s && *s != ' '; s++) ;
-		if (*s) ++s;
-		for (t = buf; *t++ = *s++; ) ;
-	    }
-	}
+	int tnum = glyph_to_trap(glyph);
+	Strcpy(buf, defsyms[
+	    trap_to_defsym(Hallucination ? rn2(TRAPNUM-3)+3 : tnum)].explanation);
     }
     else if(!glyph_is_cmap(glyph))
 	Strcpy(buf,"dark part of a room");
     else switch(glyph_to_cmap(glyph)) {
     case S_altar:
-        if(!In_endgame(&u.uz))
+	if(!In_endgame(&u.uz))
 	    Sprintf(buf, "%s altar",
 		align_str(Amask2align(levl[x][y].altarmask & ~AM_SHRINE)));
 	else Sprintf(buf, "aligned altar");
 	break;
     case S_ndoor:
-	if((levl[x][y].doormask & ~D_TRAPPED) == D_BROKEN)
+	if (is_drawbridge_wall(x, y) >= 0)
+	    Strcpy(buf,"open drawbridge portcullis");
+	else if ((levl[x][y].doormask & ~D_TRAPPED) == D_BROKEN)
 	    Strcpy(buf,"broken door");
 	else
 	    Strcpy(buf,"doorway");
 	break;
+    case S_cloud:
+	Strcpy(buf, Is_airlevel(&u.uz) ? "cloudy area" : "fog/vapor cloud");
+	break;
     default:
 	Strcpy(buf,defsyms[glyph_to_cmap(glyph)].explanation);
 	break;
@@ -156,53 +158,60 @@ lookat(x, y, buf)
  * whole name (user_typed_name == TRUE), or we've found a possible match
  * with a character/glyph and flags.help is TRUE.
  *
- * NOTE: when (user_typed_name == FALSE), inp is considered read-only and 
- *	 must not be changed directly, e.g. via lcase(). Permitted are
- *	 functions, e.g. makesingular(), which operate on a copy of inp.
+ * NOTE: when (user_typed_name == FALSE), inp is considered read-only and
+ *	 must not be changed directly, e.g. via lcase(). We want to force
+ *	 lcase() for data.base lookup so that we can have a clean key.
+ *	 Therefore, we create a copy of inp _just_ for data.base lookup.
  */
 static void
 checkfile(inp, user_typed_name)
     char *inp;
     boolean user_typed_name;
 {
-    FILE *fp;
-    char buf[BUFSZ];
-    char *ep;
+    dlb *fp;
+    char buf[BUFSZ], newstr[BUFSZ];
+    char *ep, *dbase_str;
     long txt_offset;
-    boolean found_in_file = FALSE;
+    int chk_skip;
+    boolean found_in_file = FALSE, skipping_entry = FALSE;
 
-    fp = fopen_datafile(DATAFILE, "r");
+    fp = dlb_fopen(DATAFILE, "r");
     if (!fp) {
 	pline("Cannot open data file!");
 	return;
     }
 
-    if (!strncmp(inp, "interior of ", 12))
-	inp += 12;
-    if (!strncmp(inp, "a ", 2))
-	inp += 2;
-    else if (!strncmp(inp, "an ", 3))
-	inp += 3;
-    else if (!strncmp(inp, "the ", 4))
-	inp += 4;
-    if (!strncmp(inp, "tame ", 5))
-	inp += 5;
-    else if (!strncmp(inp, "peaceful ", 9))
-	inp += 9;
-    if (!strncmp(inp, "invisible ", 10))
-	inp += 10;
+    /* To prevent the need for entries in data.base like *ngel to account
+     * for Angel and angel, make the lookup string the same for both
+     * user_typed_name and picked name.
+     */
+    dbase_str = strcpy(newstr, inp);
+    (void) lcase(dbase_str);
+
+    if (!strncmp(dbase_str, "interior of ", 12))
+	dbase_str += 12;
+    if (!strncmp(dbase_str, "a ", 2))
+	dbase_str += 2;
+    else if (!strncmp(dbase_str, "an ", 3))
+	dbase_str += 3;
+    else if (!strncmp(dbase_str, "the ", 4))
+	dbase_str += 4;
+    if (!strncmp(dbase_str, "tame ", 5))
+	dbase_str += 5;
+    else if (!strncmp(dbase_str, "peaceful ", 9))
+	dbase_str += 9;
+    if (!strncmp(dbase_str, "invisible ", 10))
+	dbase_str += 10;
 
     /* Make sure the name is non-empty. */
-    if (*inp) {
+    if (*dbase_str) {
 	/* adjust the input to remove "named " and convert to lower case */
 	char *alt = 0;	/* alternate description */
-	if ((ep = strstri(inp, " named ")) != 0)
+	if ((ep = strstri(dbase_str, " named ")) != 0)
 	    alt = ep + 7;
 	else
-	    ep = strstri(inp, " called ");
+	    ep = strstri(dbase_str, " called ");
 	if (ep) *ep = '\0';
-	if (user_typed_name)
-	    (void) lcase(inp);
 
 	/*
 	 * If the object is named, then the name is the alternate description;
@@ -213,30 +222,41 @@ checkfile(inp, user_typed_name)
 	 */
 
 	if (!alt)
-	    alt = makesingular(inp);
+	    alt = makesingular(dbase_str);
 	else
 	    if (user_typed_name)
-	    	(void) lcase(alt);
+		(void) lcase(alt);
 
 	/* skip first record; read second */
 	txt_offset = 0L;
-	if (!fgets(buf, BUFSZ, fp) || !fgets(buf, BUFSZ, fp)) {
+	if (!dlb_fgets(buf, BUFSZ, fp) || !dlb_fgets(buf, BUFSZ, fp)) {
 	    impossible("can't read 'data' file");
-	    (void) fclose(fp);
+	    (void) dlb_fclose(fp);
 	    return;
 	} else if (sscanf(buf, "%8lx\n", &txt_offset) < 1 || txt_offset <= 0)
 	    goto bad_data_file;
 
 	/* look for the appropriate entry */
-	while (fgets(buf,BUFSZ,fp)) {
+	while (dlb_fgets(buf,BUFSZ,fp)) {
 	    if (*buf == '.') break;  /* we passed last entry without success */
 
-	    if (!digit(*buf)) {
+	    if (digit(*buf)) {
+		/* a number indicates the end of current entry */
+		skipping_entry = FALSE;
+	    } else if (!skipping_entry) {
 		if (!(ep = index(buf, '\n'))) goto bad_data_file;
 		*ep = 0;
-		if (pmatch(buf, inp) || (alt && pmatch(buf, alt))) {
-		    found_in_file = TRUE;
-		    break;
+		/* if we match a key that begins with "~", skip this entry */
+		chk_skip = (*buf == '~') ? 1 : 0;
+		if (pmatch(&buf[chk_skip], dbase_str) ||
+			(alt && pmatch(&buf[chk_skip], alt))) {
+		    if (chk_skip) {
+			skipping_entry = TRUE;
+			continue;
+		    } else {
+			found_in_file = TRUE;
+			break;
+		    }
 		}
 	    }
 	}
@@ -249,25 +269,25 @@ checkfile(inp, user_typed_name)
 
 	/* skip over other possible matches for the info */
 	do {
-	    if (!fgets(buf, BUFSZ, fp)) goto bad_data_file;
+	    if (!dlb_fgets(buf, BUFSZ, fp)) goto bad_data_file;
 	} while (!digit(*buf));
 	if (sscanf(buf, "%ld,%d\n", &entry_offset, &entry_count) < 2) {
 bad_data_file:	impossible("'data' file in wrong format");
-		(void) fclose(fp);
+		(void) dlb_fclose(fp);
 		return;
 	}
 
 	if (user_typed_name || yn("More info?") == 'y') {
 	    winid datawin;
 
-	    if (fseek(fp, txt_offset + entry_offset, SEEK_SET) < 0) {
+	    if (dlb_fseek(fp, txt_offset + entry_offset, SEEK_SET) < 0) {
 		pline("? Seek error on 'data' file!");
-		(void) fclose(fp);
+		(void) dlb_fclose(fp);
 		return;
 	    }
-	    datawin = create_nhwindow(NHW_TEXT);
+	    datawin = create_nhwindow(NHW_MENU);
 	    for (i = 0; i < entry_count; i++) {
-		if (!fgets(buf, BUFSZ, fp)) goto bad_data_file;
+		if (!dlb_fgets(buf, BUFSZ, fp)) goto bad_data_file;
 		if ((ep = index(buf, '\n')) != 0) *ep = 0;
 		if (index(buf+1, '\t') != 0) (void) tabexpand(buf+1);
 		putstr(datawin, 0, buf+1);
@@ -278,7 +298,7 @@ bad_data_file:	impossible("'data' file in wrong format");
     } else if (user_typed_name)
 	pline("I don't have any information on those things.");
 
-    (void) fclose(fp);
+    (void) dlb_fclose(fp);
 }
 
 static int
@@ -286,7 +306,7 @@ do_look(quick)
     boolean quick;	/* use cursor && don't search for "more info" */
 {
     char    out_str[BUFSZ], look_buf[BUFSZ];
-    const char    *firstmatch = 0;
+    const char *x_str, *firstmatch = 0;
     int     i;
     int     sym;		/* typed symbol or converted glyph */
     int	    found;		/* count of matching syms found */
@@ -294,12 +314,10 @@ do_look(quick)
     boolean save_verbose;	/* saved value of flags.verbose */
     boolean from_screen;	/* question from the screen */
     boolean need_to_look;	/* need to get explan. from glyph */
+    boolean hit_trap;		/* true if found trap explanation */
+    int skipped_venom = 0;	/* non-zero if we ignored "splash of venom" */
     static const char *mon_interior = "the interior of a monster";
 
-#ifdef GCC_WARN
-    sym = 0;
-#endif
-
     if (quick) {
 	from_screen = TRUE;	/* yes, we want to use the cursor */
     } else {
@@ -311,9 +329,10 @@ do_look(quick)
     if (from_screen) {
 	cc.x = u.ux;
 	cc.y = u.uy;
+	sym = 0;		/* gcc -Wall lint */
     } else {
 	getlin("Specify what? (type the word)", out_str);
-	if (out_str[0] == '\033')
+	if (out_str[0] == '\0' || out_str[0] == '\033')
 	    return 0;
 
 	if (out_str[1]) {	/* user typed in a complete string */
@@ -355,11 +374,11 @@ do_look(quick)
 	    if (glyph_is_cmap(glyph)) {
 		sym = showsyms[glyph_to_cmap(glyph)];
 	    } else if (glyph_is_trap(glyph)) {
-		sym = showsyms[(glyph_to_trap(glyph) == WEB) ? S_web : S_trap];
+		sym = showsyms[trap_to_defsym(glyph_to_trap(glyph))];
 	    } else if (glyph_is_object(glyph)) {
-		sym = oc_syms[objects[glyph_to_obj(glyph)].oc_class];
+		sym = oc_syms[(int)objects[glyph_to_obj(glyph)].oc_class];
 	    } else if (glyph_is_monster(glyph)) {
-		sym = monsyms[mons[glyph_to_mon(glyph)].mlet];
+		sym = monsyms[(int)mons[glyph_to_mon(glyph)].mlet];
 	    } else if (glyph_is_swallow(glyph)) {
 		sym = showsyms[glyph_to_swallow(glyph)+S_sw_tl];
 	    } else {
@@ -393,7 +412,7 @@ do_look(quick)
 	 * and looking at something other than our own symbol, then just say
 	 * "the interior of a monster".
 	 */
-	if (u.uswallow && from_screen && is_swallow_sym((uchar) sym)) {
+	if (u.uswallow && from_screen && is_swallow_sym(sym)) {
 	    if (!found) {
 		Sprintf(out_str, "%c       %s", sym, mon_interior);
 		firstmatch = mon_interior;
@@ -407,6 +426,10 @@ do_look(quick)
 	for (i = 1; i < MAXOCLASSES; i++) {
 	    if (sym == (from_screen ? oc_syms[i] : def_oc_syms[i])) {
 		need_to_look = TRUE;
+		if (from_screen && i == VENOM_CLASS) {
+		    skipped_venom++;
+		    continue;
+		}
 		if (!found) {
 		    Sprintf(out_str, "%c       %s", sym, an(objexplain[i]));
 		    firstmatch = objexplain[i];
@@ -417,24 +440,54 @@ do_look(quick)
 	    }
 	}
 
+#define is_cmap_trap(i) ((i) >= S_arrow_trap && (i) <= S_polymorph_trap)
+#define is_cmap_drawbridge(i) ((i) >= S_vodbridge && (i) <= S_hcdbridge)
+
 	/* Now check for graphics symbols */
-	for (i = 0; i < MAXPCHARS; i++) {
-	    if (sym == (from_screen ? showsyms[i] : defsyms[i].sym) &&
-						(*defsyms[i].explanation)) {
+	for (hit_trap = FALSE, i = 0; i < MAXPCHARS; i++) {
+	    x_str = defsyms[i].explanation;
+	    if (sym == (from_screen ? showsyms[i] : defsyms[i].sym) && *x_str) {
+		/* avoid "an air", "a water", or "a floor of a room" */
+		int article = (i == S_room) ? 2 :		/* 2=>"the" */
+			      !(strcmp(x_str, "air") == 0 ||	/* 1=>"an"  */
+				strcmp(x_str, "water") == 0);	/* 0=>(none)*/
+
 		if (!found) {
-		    Sprintf(out_str, "%c       %s",
-					    sym, an(defsyms[i].explanation));
-		    firstmatch = defsyms[i].explanation;
+		    if (is_cmap_trap(i)) {
+			Sprintf(out_str, "%c       a trap", sym);
+			hit_trap = TRUE;
+		    } else {
+			Sprintf(out_str, "%c       %s", sym,
+				article == 2 ? the(x_str) :
+				article == 1 ? an(x_str) : x_str);
+		    }
+		    firstmatch = x_str;
 		    found++;
-		} else if (!u.uswallow) {
-		    found += append_str(out_str, an(defsyms[i].explanation));
+		} else if (!u.uswallow && !(hit_trap && is_cmap_trap(i)) &&
+			   !(found >= 3 && is_cmap_drawbridge(i))) {
+		    found += append_str(out_str,
+					article == 2 ? the(x_str) :
+					article == 1 ? an(x_str) : x_str);
+		    if (is_cmap_trap(i)) hit_trap = TRUE;
 		}
 
-		if (i == S_altar || i == S_trap || i == S_web)
+		if (i == S_altar || is_cmap_trap(i))
 		    need_to_look = TRUE;
 	    }
 	}
 
+	/* if we ignored venom and list turned out to be short, put it back */
+	if (skipped_venom && found < 2) {
+	    x_str = objexplain[VENOM_CLASS];
+	    if (!found) {
+		Sprintf(out_str, "%c       %s", sym, an(x_str));
+		firstmatch = x_str;
+		found++;
+	    } else {
+		found += append_str(out_str, an(x_str));
+	    }
+	}
+
 	/*
 	 * If we are looking at the screen, follow multiple posibilities or
 	 * an ambigious explanation by something more detailed.
@@ -488,24 +541,30 @@ int
 doidtrap()
 {
 	register struct trap *trap;
-	register int x,y;
+	int x, y, tt;
 
-	if(!getdir(NULL)) return 0;
+	if (!getdir((char *)0)) return 0;
 	x = u.ux + u.dx;
 	y = u.uy + u.dy;
-	for(trap = ftrap; trap; trap = trap->ntrap)
-		if(trap->tx == x && trap->ty == y && trap->tseen) {
-		    if(u.dz) {
-			if(u.dz < 0 && trap->ttyp == TRAPDOOR)
-			    continue;
-		        if(u.dz > 0 && trap->ttyp == ROCKTRAP)
-			    continue;
-		    }
-		    pline("That is a%s.",
-			  traps[ Hallucination ? rn1(TRAPNUM-3, 3) :
-				trap->ttyp]);
-		    return 0;
+	for (trap = ftrap; trap; trap = trap->ntrap)
+	    if (trap->tx == x && trap->ty == y) {
+		if (!trap->tseen) break;
+		tt = trap->ttyp;
+		if (u.dz) {
+		    if (u.dz < 0 ? (tt == TRAPDOOR || tt == HOLE) :
+			    tt == ROCKTRAP) break;
 		}
+		if (Hallucination) tt = rn1(TRAPNUM-3, 3);
+		pline("That is %s%s%s.",
+		      an(defsyms[trap_to_defsym(tt)].explanation),
+		      !trap->madeby_u ? "" : (tt == WEB) ? " woven" :
+			  /* trap doors & spiked pits can't be made by
+			     player, and should be considered at least
+			     as much "set" as "dug" anyway */
+			  (tt == HOLE || tt == PIT) ? " dug" : " set",
+		      !trap->madeby_u ? "" : " by you");
+		return 0;
+	    }
 	pline("I can't see a trap there.");
 	return 0;
 }
@@ -513,11 +572,11 @@ doidtrap()
 int
 dowhatdoes()
 {
-	FILE *fp;
+	dlb *fp;
 	char bufr[BUFSZ+6];
 	register char *buf = &bufr[6], *ep, q, ctrl, meta;
 
-	fp = fopen_datafile(CMDHELPFILE, "r");
+	fp = dlb_fopen(CMDHELPFILE, "r");
 	if (!fp) {
 		pline("Cannot open data file!");
 		return 0;
@@ -526,13 +585,13 @@ dowhatdoes()
 #if defined(UNIX) || defined(VMS)
 	introff();
 #endif
-	q = yn_function("What command?", NULL, '\0');
+	q = yn_function("What command?", (char *)0, '\0');
 #if defined(UNIX) || defined(VMS)
 	intron();
 #endif
 	ctrl = ((q <= '\033') ? (q - 1 + 'A') : 0);
 	meta = ((0x80 & q) ? (0x7f & q) : 0);
-	while(fgets(buf,BUFSZ,fp))
+	while(dlb_fgets(buf,BUFSZ,fp))
 	    if ((ctrl && *buf=='^' && *(buf+1)==ctrl) ||
 		(meta && *buf=='M' && *(buf+1)=='-' && *(buf+2)==meta) ||
 		*buf==q) {
@@ -552,100 +611,105 @@ dowhatdoes()
 			(void) strncpy(buf+1, "       ", 7);
 		}
 		pline("%s", buf);
-		(void) fclose(fp);
+		(void) dlb_fclose(fp);
 		return 0;
 	    }
 	pline("I've never heard of such commands.");
-	(void) fclose(fp);
+	(void) dlb_fclose(fp);
 	return 0;
 }
 
 /* data for help_menu() */
 static const char *help_menu_items[] = {
-	"Information available:",
-	"",
-	"a.  Long description of the game and commands.",
-	"b.  List of game commands.",
-	"c.  Concise history of NetHack.",
-	"d.  Info on a character in the game display.",
-	"e.  Info on what a given key does.",
-	"f.  List of game options.",
-	"g.  Longer explanation of game options.",
-	"h.  List of extended commands.",
-	"i.  The NetHack license.",
+/* 0*/	"Long description of the game and commands.",
+/* 1*/	"List of game commands.",
+/* 2*/	"Concise history of NetHack.",
+/* 3*/	"Info on a character in the game display.",
+/* 4*/	"Info on what a given key does.",
+/* 5*/	"List of game options.",
+/* 6*/	"Longer explanation of game options.",
+/* 7*/	"List of extended commands.",
+/* 8*/	"The NetHack license.",
 #ifdef PORT_HELP
-	"j.  %s-specific help and commands.",
+	"%s-specific help and commands.",
+#define PORT_HELP_ID 100
+#define WIZHLP_SLOT 10
+#else
+#define WIZHLP_SLOT 9
 #endif
 #ifdef WIZARD
-# ifdef PORT_HELP
-# define WIZHLP_SLOT 12	/* assumed to be next to last by code below */
-	"k.  List of wizard-mode commands.",
-# else
-# define WIZHLP_SLOT 11	/* assumed to be next to last by code below */
-	"j.  List of wizard-mode commands.",
-# endif
+	"List of wizard-mode commands.",
 #endif
 	"",
-	NULL
+	(char *)0
 };
 
-static char
-help_menu()
+static boolean
+help_menu(sel)
+	int *sel;
 {
 	winid tmpwin = create_nhwindow(NHW_MENU);
 #ifdef PORT_HELP
 	char helpbuf[QBUFSZ];
 #endif
-	char hc;
-	register int i;
+	int i, n;
+	menu_item *selected;
+	anything any;
 
+	any.a_void = 0;		/* zero all bits */
 	start_menu(tmpwin);
 #ifdef WIZARD
 	if (!wizard) help_menu_items[WIZHLP_SLOT] = "",
-		     help_menu_items[WIZHLP_SLOT+1] = NULL;
+		     help_menu_items[WIZHLP_SLOT+1] = (char *)0;
 #endif
 	for (i = 0; help_menu_items[i]; i++)
 #ifdef PORT_HELP
 	    /* port-specific line has a %s in it for the PORT_ID */
-	    if (index(help_menu_items[i], '%')) {
+	    if (help_menu_items[i][0] == '%') {
 		Sprintf(helpbuf, help_menu_items[i], PORT_ID);
-		add_menu(tmpwin, helpbuf[0], 0, helpbuf);
+		any.a_int = PORT_HELP_ID + 1;
+		add_menu(tmpwin, NO_GLYPH, &any, 0, 0, ATR_NONE,
+			 helpbuf, MENU_UNSELECTED);
 	    } else
 #endif
 	    {
-		add_menu(tmpwin, i ? *help_menu_items[i] : 0, 0,
-			 help_menu_items[i]);
+		any.a_int = (*help_menu_items[i]) ? i+1 : 0;
+		add_menu(tmpwin, NO_GLYPH, &any, 0, 0,
+			ATR_NONE, help_menu_items[i], MENU_UNSELECTED);
 	    }
-	end_menu(tmpwin, '\033', "\033", "Select one item or ESC: ");
-	hc = select_menu(tmpwin);
+	end_menu(tmpwin, "Select one item:");
+	n = select_menu(tmpwin, PICK_ONE, &selected);
 	destroy_nhwindow(tmpwin);
-	return hc;
+	if (n > 0) {
+	    *sel = selected[0].item.a_int - 1;
+	    free((genericptr_t)selected);
+	    return TRUE;
+	}
+	return FALSE;
 }
 
 int
 dohelp()
 {
-	char hc = help_menu();
-	if (!index(quitchars, hc)) {
-		switch(hc) {
-			case 'a':  display_file(HELP, TRUE);  break;
-			case 'b':  display_file(SHELP, TRUE);  break;
-			case 'c':  (void) dohistory();  break;
-			case 'd':  (void) dowhatis();  break;
-			case 'e':  (void) dowhatdoes();  break;
-			case 'f':  option_help();  break;
-			case 'g':  display_file(OPTIONFILE, TRUE);  break;
-			case 'h':  (void) doextlist();  break;
-			case 'i':  display_file(LICENSE, TRUE);  break;
+	int sel = 0;
+
+	if (help_menu(&sel)) {
+		switch (sel) {
+			case  0:  display_file(HELP, TRUE);  break;
+			case  1:  display_file(SHELP, TRUE);  break;
+			case  2:  (void) dohistory();  break;
+			case  3:  (void) dowhatis();  break;
+			case  4:  (void) dowhatdoes();  break;
+			case  5:  option_help();  break;
+			case  6:  display_file(OPTIONFILE, TRUE);  break;
+			case  7:  (void) doextlist();  break;
+			case  8:  display_file(LICENSE, TRUE);  break;
+#ifdef WIZARD
+			/* handle slot 9 or 10 */
+			default: display_file(DEBUGHELP, TRUE);  break;
+#endif
 #ifdef PORT_HELP
-			case 'j':  port_help();  break;
-# ifdef WIZARD
-			case 'k':  display_file(DEBUGHELP, TRUE);  break;
-# endif
-#else
-# ifdef WIZARD
-			case 'j':  display_file(DEBUGHELP, TRUE);  break;
-# endif
+			case PORT_HELP_ID:  port_help();  break;
 #endif
 		}
 	}
