@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)write.c	3.1	91/01/04
+/*	SCCS Id: @(#)write.c	3.2	96/05/05	*/
 /* NetHack may be freely redistributed.  See license for details. */
 
 #include "hack.h"
@@ -16,7 +16,7 @@ register struct obj *otmp;
 	if (otmp->oclass == SPBOOK_CLASS)
 		return(10 * objects[otmp->otyp].oc_level);
 
-	switch(otmp->otyp)  {
+	switch (otmp->otyp) {
 # ifdef MAIL
 	case SCR_MAIL:
 		return(2);
@@ -69,68 +69,95 @@ dowrite(pen)
 register struct obj *pen;
 {
 	register struct obj *paper;
-	char namebuf[BUFSZ], scrbuf[BUFSZ];
+	char namebuf[BUFSZ], *nm, *bp;
 	register struct obj *new_obj;
 	int basecost, actualcost;
 	int curseval;
 	char qbuf[QBUFSZ];
-	
-	if(!pen)
-		return(0);
-	/* already tested before only call of dowrite() (from doapply())
-	if(pen->otyp != MAGIC_MARKER)  {
-		You("can't write with that!");
-		return(0);
+	int first, last, i;
+	boolean by_descr = FALSE;
+	const char *typeword;
+
+	if (nohands(uasmon)) {
+	    You("need hands to be able to write!");
+	    return 0;
+	} else if (Glib) {
+	    dropx(pen);
+	    pline("%s slips from your %s.", The(xname(pen)),
+			makeplural(body_part(FINGER)));
+	    return 1;
 	}
-	*/
 
 	/* get paper to write on */
 	paper = getobj(write_on,"write on");
 	if(!paper)
 		return(0);
+	typeword = (paper->oclass == SPBOOK_CLASS) ? "spellbook" : "scroll";
 	if(Blind && !paper->dknown) {
-		You("don't know if that %s is blank or not!",
-		      paper->oclass == SPBOOK_CLASS ? "spellbook" :
-		      "scroll");
+		You("don't know if that %s is blank or not!", typeword);
 		return(1);
 	}
 	paper->dknown = 1;
 	if(paper->otyp != SCR_BLANK_PAPER && paper->otyp != SPE_BLANK_PAPER) {
-		pline("That %s is not blank!",
-		    paper->oclass == SPBOOK_CLASS ? "spellbook" :
-		    "scroll");
+		pline("That %s is not blank!", typeword);
 		exercise(A_WIS, FALSE);
 		return(1);
 	}
 
 	/* what to write */
-	Sprintf(qbuf, "What type of %s do you want to write? ",
-	      paper->oclass == SPBOOK_CLASS ? "spellbook" :
-	      "scroll");
+	Sprintf(qbuf, "What type of %s do you want to write?", typeword);
 	getlin(qbuf, namebuf);
+	(void)mungspaces(namebuf);	/* remove any excess whitespace */
 	if(namebuf[0] == '\033' || !namebuf[0])
 		return(1);
-	scrbuf[0] = '\0';
-	if (paper->oclass == SPBOOK_CLASS) {
-		if(strncmp(namebuf,"spellbook of ",13) != 0)
-			Strcpy(scrbuf,"spellbook of ");
-	}
-	else if(strncmp(namebuf,"scroll of ",10) != 0)
-		Strcpy(scrbuf,"scroll of ");
-	Strcat(scrbuf,namebuf);
-	new_obj = readobjnam(scrbuf);
+	nm = namebuf;
+	if (!strncmpi(nm, "scroll ", 7)) nm += 7;
+	else if (!strncmpi(nm, "spellbook ", 10)) nm += 10;
+	if (!strncmpi(nm, "of ", 3)) nm += 3;
 
+	if ((bp = strstri(nm, " armour")) != 0) {
+		(void)strncpy(bp, " armor ", 7);	/* won't add '\0' */
+		(void)mungspaces(bp + 1);	/* remove the extra space */
+	}
+
+	first = bases[(int)paper->oclass];
+	last = bases[(int)paper->oclass + 1] - 1;
+	for (i = first; i <= last; i++) {
+		/* extra shufflable descr not representing a real object */
+		if (!OBJ_NAME(objects[i])) continue;
+
+		if (!strcmpi(OBJ_NAME(objects[i]), nm))
+			goto found;
+		if (!strcmpi(OBJ_DESCR(objects[i]), nm)) {
+			by_descr = TRUE;
+			goto found;
+		}
+	}
+
+	pline("There is no such %s!", typeword);
+	return 1;
+found:
+
+	if (i == SCR_BLANK_PAPER || i == SPE_BLANK_PAPER) {
+		You_cant("write that!");
+		pline("It's obscene!");
+		return 1;
+	} else if (i == SPE_BOOK_OF_THE_DEAD) {
+		pline("No mere dungeon adventurer could write that.");
+		return 1;
+	} else if (by_descr && paper->oclass == SPBOOK_CLASS &&
+		    !objects[i].oc_name_known) {
+		/* can't write unknown spellbooks by description */
+		pline(
+		  "Unfortunately you don't have enough information to go on.");
+		return 1;
+	}
+
+	new_obj = mksobj(i, FALSE, FALSE);
 	new_obj->bknown = (paper->bknown && pen->bknown);
 
-	if((new_obj->oclass != SCROLL_CLASS ||
-	              new_obj->otyp == SCR_BLANK_PAPER)
-	    && (new_obj->oclass != SPBOOK_CLASS || 
-                      new_obj->otyp == SPE_BLANK_PAPER)) {
-		You("can't write that!");
-		pline("It's obscene!");
-		obfree(new_obj, (struct obj *) 0); /* pb@ethz.uucp */
-		return(1);
-	}
+	/* shk imposes a flat rate per use, not based on actual charges used */
+	check_unpaid(pen);
 
 	/* see if there's enough ink */
 	basecost = cost(new_obj);
@@ -150,9 +177,10 @@ register struct obj *pen;
 		Your("marker dries out!");
 		/* scrolls disappear, spellbooks don't */
 		if (paper->oclass == SPBOOK_CLASS)
-			pline("The spellbook is left unfinished.");
+			pline_The(
+		       "spellbook is left unfinished and your writing fades.");
 		else {
-			pline("The scroll is now useless and disappears!");
+			pline_The("scroll is now useless and disappears!");
 			useup(paper);
 		}
 		pen->spe = 0;
@@ -162,15 +190,21 @@ register struct obj *pen;
 	pen->spe -= actualcost;
 
 	/* can't write if we don't know it - unless we're lucky */
-	if(!(objects[new_obj->otyp].oc_name_known) && 
-	   !(objects[new_obj->otyp].oc_uname) && 
-	   (rnl(pl_character[0] == 'W' ? 3 : 15))) {
-		You("don't know how to write that!");
+	if(!(objects[new_obj->otyp].oc_name_known) &&
+	   !(objects[new_obj->otyp].oc_uname) &&
+	   (rnl(Role_is('W') ? 3 : 15))) {
+		You("%s to write that!", by_descr ? "fail" : "don't know how");
 		/* scrolls disappear, spellbooks don't */
 		if (paper->oclass == SPBOOK_CLASS)
-			You("write in your best handwriting:  \"My Diary\".");
+			You(
+       "write in your best handwriting:  \"My Diary\", but it quickly fades.");
 		else {
-			You("write \"%s was here!\" and the scroll disappears.",plname);
+			if (by_descr) {
+			    Strcpy(namebuf, OBJ_DESCR(objects[new_obj->otyp]));
+			    wipeout_text(namebuf, (6+MAXULEV - u.ulevel)/6, 0);
+			} else
+			    Sprintf(namebuf, "%s was here!", plname);
+			You("write \"%s\" and the scroll disappears.", namebuf);
 			useup(paper);
 		}
 		obfree(new_obj, (struct obj *) 0);
@@ -180,10 +214,12 @@ register struct obj *pen;
 	/* useup old scroll / spellbook */
 	useup(paper);
 
-	/* now you know it! */
-	makeknown(new_obj->otyp);
-
 	/* success */
+	if (new_obj->oclass == SPBOOK_CLASS) {
+		/* acknowledge the change in the object's description... */
+		pline("The spellbook warps strangely, then turns %s.",
+		      OBJ_DESCR(objects[new_obj->otyp]));
+	}
 	new_obj->blessed = (curseval > 0);
 	new_obj->cursed = (curseval < 0);
 #ifdef MAIL
@@ -192,7 +228,6 @@ register struct obj *pen;
 	new_obj = hold_another_object(new_obj, "Oops!  %s out of your grasp!",
 					       The(aobjnam(new_obj, "slip")),
 					       (const char *)0);
-	if (new_obj) new_obj->known = 1;
 	return(1);
 }
 
