@@ -18,7 +18,7 @@
  * SIZE (-1) info: flags: 0x5880, size: 65536L/65536L (64k/64k)
  * libraries: MacTraps [yes], MacTraps2 (HFileStuff) [yes], ANSI [no]
  * compatibility: system 6 and system 7
- * misc: sizeof(int): 4, "\p": unsigned char, enum size varies,
+ * misc: sizeof(int): 2, "\p": unsigned char, enum size varies,
  *   prototypes required, type checking enforced, no optimizers,
  *   FAR CODE [no], FAR DATA [no], SEPARATE STRS [no], single segment,
  *   short macsbug symbols
@@ -36,10 +36,6 @@
  * - Share AppleEvents with NetHack to auto-recover crashed games.
  */
 
-#if 1
-/************************************************************************\
- * (1) precompile header => mrecover.h, (0) compile code
-\************************************************************************/
 
 /**** Toolbox defines ****/
 
@@ -50,7 +46,9 @@
 #include <OSUtils.h>
 #include <Resources.h>
 #include <Files.h>
+#ifdef applec
 #include <SysEqu.h>
+#endif
 #include <SegLoad.h>
 
 #include <Quickdraw.h>
@@ -67,8 +65,9 @@
 #include <Script.h>
 #include <StandardFile.h>
 #include <ToolUtils.h>
+#include <Processes.h>
 
-#if 1	/* glue for System 7 Icon Family call (needed by Think C 5.0.4) */
+#ifndef __MWERKS__	/* glue for System 7 Icon Family call (needed by Think C 5.0.4) */
 pascal OSErr GetIconSuite(Handle *theIconSuite, short theResID, long selector)
 	= {0x303C, 0x0501, 0xABC9};
 #endif
@@ -198,12 +197,7 @@ typedef struct versXRec
 	unsigned char	versStr[];	/* (small string)(large string) */
 } versXRec, *versXPtr, **versXHandle;
 
-#else
-/************************************************************************\
- * compile source code
-\************************************************************************/
 
-#include "mrecover.h"
 
 /**** Global variables ****/
 modeFlags		in = {1};				/* in Front */
@@ -228,6 +222,8 @@ Point			sfGetWhere;				/* top left corner of get file dialog */
 Ptr				pIOBuf;					/* read/write buffer pointer */
 short			vRefNum;				/* SFGetFile working directory/volume refnum */
 long			dirID;					/* directory i.d. */
+NMUPP			nmCompletionUPP;		/* UPP for nmCompletion */
+FileFilterUPP	basenameFileFilterUPP;	/* UPP for basenameFileFilter */
 
 #define CREATOR		'nh31'				/* NetHack signature */
 #define SAVETYPE	'SAVE'				/* save file type */
@@ -244,7 +240,7 @@ typedef schar		xchar;				/* global.h */
 unsigned char	savename[SAVESIZE];		/* originally a C string */
 unsigned char	lock[256];				/* pascal string */
 
-long			hpid;					/* NetHack (unix-style) process i.d. */
+int			hpid;					/* NetHack (unix-style) process i.d. */
 short			saveRefNum;				/* save file descriptor */
 short			gameRefNum;				/* level 0 file descriptor */
 short			levRefNum;				/* level n file descriptor */
@@ -300,15 +296,17 @@ main()
 	InitWindows();
 	InitMenus();
 	TEInit();
-	InitDialogs((ResumeProcPtr) 0);
+	InitDialogs(0L);
 	InitCursor();
+	nmCompletionUPP = NewNMProc(nmCompletion);
+	basenameFileFilterUPP = NewFileFilterProc(basenameFileFilter);
 
 	/* get system environment, notification requires 6.0 or better */
 	(void) SysEnvirons(curSysEnvVers, &sysEnv);
 	if (sysEnv.systemVersion < 0x0600)
 	{
 		ParamText("\pAbort: System 6.0 is required", "\p", "\p", "\p");
-		(void) Alert(alidNote, (ModalFilterProcPtr) 0L);
+		(void) Alert(alidNote, (ModalFilterUPP) 0L);
 		ExitToShell();
 	}
 
@@ -333,6 +331,7 @@ warmup()
 				in.Front = (wnEvt.message & resumeFlag);
 	}
 
+#if 0 // ???
 	/* clear out the Finder info */
 	{
 		short	message, count;
@@ -341,15 +340,30 @@ warmup()
 		while(count)
 			ClrAppFiles(count--);
 	}
+#endif
 
 	/* fill out the notification template */
 	nmt.nmr.qType = nmType;
 	nmt.nmr.nmMark = 1;
 	nmt.nmr.nmSound = (Handle) -1L;		/* system beep */
 	nmt.nmr.nmStr = nmt.nmBuf;
-	nmt.nmr.nmResp = nmCompletion;
+	nmt.nmr.nmResp = nmCompletionUPP;
 	nmt.nmr.nmPending = (long) &in.Notify;
 
+
+#if 1
+	{
+		/* get the app name */
+		ProcessInfoRec info;
+		ProcessSerialNumber psn;
+
+		info.processInfoLength = sizeof(info);
+		info.processName = nmt.nmBuf;
+		info.processAppSpec = NULL;
+		GetCurrentProcess(&psn);
+		GetProcessInformation(&psn, &info);
+	}
+#else
 	/* prepend app name (31 chars or less) to notification buffer */
 	{
 		short	apRefNum;
@@ -357,6 +371,7 @@ warmup()
 
 		GetAppParms(* (Str255 *) &nmt.nmBuf, &apRefNum, &apParams);
 	}
+#endif
 
 	/* add formatting (two line returns) */
 	nmt.nmBuf[++(nmt.nmBuf[0])] = '\r';
@@ -574,7 +589,7 @@ note(short errorSignal, short alertID, unsigned char *msg)
 
 	/* in front and no error so use an alert */
 	ParamText(msg, "\p", "\p", "\p");
-	(void) Alert(alertID, (ModalFilterProcPtr) 0L);
+	(void) Alert(alertID, (ModalFilterUPP) 0L);
 	ResetAlrtStage();
 
 	memActivity++;
@@ -988,8 +1003,8 @@ beginRecover()
 	SFTypeList		levlType = {'LEVL'};
 	SFReply			sfGetReply;
 
-	SFGetFile(sfGetWhere, "\p", &basenameFileFilter, 1, levlType,
-				(DlgHookProcPtr) 0L, &sfGetReply);
+	SFGetFile(sfGetWhere, "\p", basenameFileFilterUPP, 1, levlType,
+				(DlgHookUPP) 0L, &sfGetReply);
 
 	memActivity++;
 
@@ -1112,7 +1127,7 @@ saveRezStrings()
 	{
 		plName++;
 		(*savename)--;
-		hpid /= 10L;
+		hpid /= 10;
 	}
 	while (hpid);
 	*plName = *savename;
@@ -1256,7 +1271,7 @@ copy_bytes(short inRefNum, short outRefNum)
 static void
 restore_savefile()
 {
-	static long	savelev;
+	static int	savelev;
 	long		saveTemp, lev;
 	xchar		levc;
 
@@ -1402,4 +1417,4 @@ unlink_file(unsigned char *filename)
 		return;
 	}
 }
-#endif
+
