@@ -1,6 +1,6 @@
-/*	SCCS Id: @(#)msdos.c	 3.1	 93/05/06		  */
-/* Copyright (c) NetHack PC Development Team 1990, 1991, 1992	  */
-/* NetHack may be freely redistributed.  See license for details. */
+/*	SCCS Id: @(#)msdos.c	 3.2	 94/02/19		          */
+/* Copyright (c) NetHack PC Development Team 1990, 1991, 1992, 1993, 1994 */
+/* NetHack may be freely redistributed.  See license for details.         */
 
 /*
  *  MSDOS system functions.
@@ -12,6 +12,7 @@
 #include "hack.h"
 
 #ifdef MSDOS
+#include "pcvideo.h"
 
 #include <dos.h>
 #include <ctype.h>
@@ -31,7 +32,7 @@
 /*
  * BIOS interrupts
  */
-#ifdef PC9801
+#ifdef PC9800
 #define KEYBRD_BIOS 0x18
 #else
 #define KEYBRD_BIOS 0x16
@@ -42,6 +43,7 @@
  */
 #define READCHAR    0x00    /* Read Character from Keyboard */
 #define GETKEYFLAGS 0x02    /* Get Keyboard Flags */
+/*#define KEY_DEBUG	    /* print values of unexpected key codes - devel*/
 
 void FDECL(get_cursor,(int *, int *));
 
@@ -53,6 +55,9 @@ static char NDECL(DOSgetch);
 static char NDECL(BIOSgetch);
 static char * NDECL(getdta);
 static unsigned int FDECL(dos_ioctl, (int,int,unsigned));
+#ifdef USE_TILES
+extern boolean FDECL(pckeys,(unsigned char, unsigned char));	/* pckeys.c */
+#endif
 
 int
 tgetch()
@@ -60,10 +65,16 @@ tgetch()
 	char ch;
 
 	/* BIOSgetch can use the numeric key pad on IBM compatibles. */
+# ifdef SIMULATE_CURSOR
+	if (flags.grmode && cursor_flag) DrawCursor();
+# endif
 	if (flags.BIOS)
 		ch = BIOSgetch();
 	else
 		ch = DOSgetch();
+# ifdef SIMULATE_CURSOR
+	if (flags.grmode && cursor_flag) HideCursor();
+# endif
 	return ((ch == '\r') ? '\n' : ch);
 }
 
@@ -72,7 +83,7 @@ tgetch()
 /*
  *  Keyboard translation tables.
  */
-#ifdef PC9801
+#ifdef PC9800
 #define KEYPADLO	0x38
 #define KEYPADHI	0x50
 #else
@@ -91,7 +102,7 @@ tgetch()
 static const struct pad {
 	char normal, shift, cntrl;
 } keypad[PADKEYS] = {
-#ifdef PC9801
+#ifdef PC9800
 			{'>', '>', '>'},		/* Ins */
 			{'<', '<', '<'},		/* Del */
 			{'k', 'K', C('k')},		/* Up */
@@ -115,7 +126,7 @@ static const struct pad {
 			{'n', 'N', C('n')},		/* 3 */
 			{'=', '=', '='},		/* = */
 			{'i', 'I', C('i')},		/* 0 */
-			{',' ':', ':'}, 		/* , */
+			{',', ':', ':'}, 		/* , */
 			{'.', '.', '.'} 		/* . */
 #else
 			{'y', 'Y', C('y')},		/* 7 */
@@ -133,7 +144,7 @@ static const struct pad {
 			{'.', ':', ':'}			/* Del */
 #endif
 }, numpad[PADKEYS] = {
-#ifdef PC9801
+#ifdef PC9800
 			{'>', '>', '>'},		/* Ins */
 			{'<', '<', '<'},		/* Del */
 			{'8', M('8'), '8'},		/* Up */
@@ -184,14 +195,14 @@ static const struct pad {
  * scan code table to translate the scan code into a letter, then set the
  * "meta" bit for it.  -3.
  */
-#ifdef PC9801
+#ifdef PC9800
 #define SCANLO		0x5
 #else
 #define SCANLO		0x10
-#endif /* PC9801 */
+#endif /* PC9800 */
 
 static const char scanmap[] = { 	/* ... */
-#ifdef PC9801
+#ifdef PC9800
 			 0,  0,  0,  0,  0,  0, '-','^','\\','\b',
 	'\t','q','w','e','r','t','y','u','i','o','p','@','[', '\n',
 	'a','s','d','f','g','h','j','k','l',';',':', ']',
@@ -200,7 +211,7 @@ static const char scanmap[] = { 	/* ... */
 	'q','w','e','r','t','y','u','i','o','p','[',']', '\n',
 	0, 'a','s','d','f','g','h','j','k','l',';','\'', '`',
 	0, '\\', 'z','x','c','v','b','n','m',',','.','?'	/* ... */
-#endif /* PC9801 */
+#endif /* PC9800 */
 };
 
 #define inmap(x)	(SCANLO <= (x) && (x) < SCANLO + SIZE(scanmap))
@@ -208,7 +219,7 @@ static const char scanmap[] = { 	/* ... */
 /*
  * BIOSgetch gets keys directly with a BIOS call.
  */
-#ifdef PC9801
+#ifdef PC9800
 #define SHIFT		0x1
 #define KANA		0x4
 #define GRPH		0x8
@@ -217,16 +228,16 @@ static const char scanmap[] = { 	/* ... */
 #define SHIFT		(0x1 | 0x2)
 #define CTRL		0x4
 #define ALT		0x8
-#endif /* PC9801 */
+#endif /* PC9800 */
 
 static char
 BIOSgetch()
 {
-	unsigned char scan, shift, ch;
-	const struct pad *kpad;
-
-	union REGS regs;
+      unsigned char scan, shift, ch=0;
+      const struct pad *kpad;
+      union REGS regs;
 
+      do {
 	/* Get scan code.
 	 */
 	regs.h.ah = READCHAR;
@@ -249,8 +260,15 @@ BIOSgetch()
 		else
 			ch = kpad[scan - KEYPADLO].normal;
 	}
+#ifdef USE_TILES
+	/* Check for special interface manipulation keys */
+	if (pckeys(scan, shift)) {
+		ch = 0xFF;
+		continue;
+	}
+#endif
 	/* Translate unassigned Alt-letters */
-#ifdef PC9801
+#ifdef PC9800
 	if (shift & KANA)
 		return 0;
 	if ((shift & GRPH) && (ch >= 0x80)) {
@@ -261,7 +279,8 @@ BIOSgetch()
 			ch = scanmap[scan - SCANLO];
 		return (isprint(ch) ? M(ch) : ch);
 	}
-	return ch;
+      } while (ch == 0xFF);
+      return ch;
 }
 
 static char
@@ -275,7 +294,7 @@ DOSgetch()
 	intdos(&regs, &regs);
 	ch = regs.h.al;
 
-#ifdef PC9801
+#ifdef PC9800
 	if (ch < 0)	/* KANA letters and GRPH-shifted letters(?) */
 		ch = 0; /* munch it */
 #else
@@ -334,7 +353,7 @@ char *path;
  * Functions to get filenames using wildcards
  */
 int
-findfirst(path)
+findfirst_file(path)
 char *path;
 {
 	union REGS regs;
@@ -349,7 +368,7 @@ char *path;
 }
 
 int
-findnext() {
+findnext_file() {
 	union REGS regs;
 
 	regs.h.ah = FINDNEXT;
@@ -375,7 +394,7 @@ getdta()
 	regs.h.ah = GETDTA;
 	intdosx(&regs, &regs, &sregs);
 #   ifdef MK_FP
-	ret = MK_FP(sregs.es, regs.x.bx);
+	ret = (char *)MK_FP(sregs.es, regs.x.bx);
 #   else
 	FP_OFF(ret) = regs.x.bx;
 	FP_SEG(ret) = sregs.es;
@@ -384,12 +403,12 @@ getdta()
 }
 
 long
-filesize(file)
+filesize_nh(file)
 char *file;
 {
 	char *dta;
 
-	if (findfirst(file)) {
+	if (findfirst_file(file)) {
 		dta = getdta();
 		return  (* (long *) (dta + 26));
 	} else
@@ -410,7 +429,7 @@ char *str;
 	union REGS inregs;
 	char drive;
 
-	if ((ptr = index(str, ':')) != NULL) {
+	if ((ptr = index(str, ':')) != (char *)0) {
 		drive = toupper(*(ptr - 1));
 		inregs.h.ah = SELECTDISK;
 		inregs.h.dl = drive - 'A';
