@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)winmesg.c	3.1	93/02/02		  */
+/*	SCCS Id: @(#)winmesg.c	3.2	96/04/05	*/
 /* Copyright (c) Dean Luick, 1992				  */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -45,6 +45,11 @@ static void FDECL(mesg_exposed, (Widget,XtPointer,XtPointer));
 static void FDECL(get_gc, (Widget,struct mesg_info_t *));
 static void FDECL(mesg_resized, (Widget,XtPointer,XtPointer));
 
+static char mesg_translations[] =
+"#override\n\
+ <Key>:		input()	\
+";
+
 /* Move the message window's vertical scrollbar's slider to the bottom. */
 void
 set_message_slider(wp)
@@ -85,18 +90,8 @@ create_message_window(wp, create_popup, parent)
     mesg_info->dirty = False;
     mesg_info->viewport_width = mesg_info->viewport_height = 0;
 
-    /*
-     * We should have an .Xdefaults option that specifies the number of lines
-     * to be displayed.  Until then, we'll use DEFAULT_LINES_DISPLAYED.
-     * E.g.:
-     *
-     *	if (a lines value from .Xdefaults exists)
-     *	    lines_displayed = lines value from .Xdefaults;
-     *	else
-     *	    lines_displayed = DEFAULT_LINES_DISPLAYED;
-     */
-    if (flags.msg_history < DEFAULT_LINES_DISPLAYED)
-	flags.msg_history = DEFAULT_LINES_DISPLAYED;
+    if (flags.msg_history < appResources.message_lines)
+	flags.msg_history = appResources.message_lines;
     if (flags.msg_history > MAX_HISTORY)	/* a sanity check */
 	flags.msg_history = MAX_HISTORY;
 
@@ -135,6 +130,10 @@ create_message_window(wp, create_popup, parent)
      * we know what font we are using.
      */
     num_args = 0;
+    if (!create_popup) {
+	XtSetArg(args[num_args], XtNtranslations,
+		 XtParseTranslationTable(mesg_translations));	num_args++;
+    }
     wp->w = XtCreateManagedWidget(
 		"message",		/* name */
 		windowWidgetClass,	/* widget class from Window.h */
@@ -146,7 +145,7 @@ create_message_window(wp, create_popup, parent)
 
     /*
      * Now adjust the height and width of the message window so that it
-     * is DEFAULT_LINES_DISPLAYED high and DEFAULT_MESSAGE_WIDTH wide.
+     * is appResources.message_lines high and DEFAULT_MESSAGE_WIDTH wide.
      */
 
     /* Get the font information. */
@@ -182,7 +181,7 @@ create_message_window(wp, create_popup, parent)
     /* make sure viewport height makes sense before realizing it */
     num_args = 0;
     mesg_info->viewport_height =
-	DEFAULT_LINES_DISPLAYED * mesg_info->char_height;
+	appResources.message_lines * mesg_info->char_height;
     XtSetArg(args[num_args], XtNheight, mesg_info->viewport_height);num_args++;
     XtSetValues(viewport, args, num_args);
 
@@ -206,11 +205,17 @@ destroy_message_window(wp)
 {
     if (wp->popup) {
 	nh_XtPopdown(wp->popup);
-	XtDestroyWidget(wp->popup);
-	set_circle_buf(wp->mesg_information, 0);	/* free buffer list */
-	free((char *)wp->mesg_information);
+	if (!wp->keep_window)
+	    XtDestroyWidget(wp->popup),  wp->popup = (Widget)0;
     }
-    wp->type = NHW_NONE;
+    if (wp->mesg_information) {
+	set_circle_buf(wp->mesg_information, 0);	/* free buffer list */
+	free((genericptr_t)wp->mesg_information),  wp->mesg_information = 0;
+    }
+    if (wp->keep_window)
+	XtRemoveCallback(wp->w, XtNexposeCallback, mesg_exposed, (XtPointer)0);
+    else
+	wp->type = NHW_NONE;
 }
 
 
@@ -286,15 +291,15 @@ set_circle_buf(mesg_info, count)
     if (count < mesg_info->num_lines) {
 	/*
 	 * Toss num_lines - count line entries from our circular list.
-	 *  
+	 *
 	 * We lose lines from the front (top) of the list.  We _know_
 	 * the list is non_empty.
 	 */
 	tail = get_previous(mesg_info->head);
-	for (i = mesg_info->num_lines - count;	i--; ) {
+	for (i = mesg_info->num_lines - count; i > 0; i--) {
 	    curr = mesg_info->head;
 	    mesg_info->head = curr->next;
-	    if (curr->line) free(curr->line);
+	    if (curr->line) free((genericptr_t)curr->line);
 	    free((genericptr_t)curr);
 	}
 	if (count == 0) {
@@ -391,7 +396,7 @@ add_line(mesg_info, s)
 	if (curr->line) free(curr->line);	/* free old line */
 
 	curr->buf_length = new_line_length + 1;
-	curr->line = (char *) alloc(curr->buf_length);
+	curr->line = (char *) alloc((unsigned)curr->buf_length);
     }
 
     Strcpy(curr->line, s);			/* copy info */
@@ -542,6 +547,7 @@ mesg_exposed(w, client_data, widget_data)
 	while (XCheckTypedWindowEvent(dpy, win, Expose, &evt)) continue;
 
 	wp = find_widget(w);
+	if (wp->keep_window && !wp->mesg_information) return;
 	mesg_check_size_change(wp);
 	redraw_message_window(wp);
     }
@@ -572,7 +578,7 @@ get_gc(w, mesg_info)
 /*
  * Handle resizes on a message window.  Correct saved pixel height and width.
  * Adjust circle buffer to accomidate the new size.
- * 
+ *
  * Problem:  If the resize decreases the width of the window such that
  * some lines are now longer than the window, they will be cut off by
  * X itself.  All new lines will be split to the new size, but the ends
@@ -608,10 +614,12 @@ mesg_resized(w, client_data, call_data)
 			(int) pixel_height / wp->mesg_information->char_height);
 
 #ifdef VERBOSE
-    printf("Message resize.  Pixel: width = %d, height = %d;  Lines: old = %d, new = %d\n", 
+    printf("Message resize.  Pixel: width = %d, height = %d;  Lines: old = %d, new = %d\n",
 	pixel_width,
 	pixel_height,
 	old_lines,
 	wp->mesg_information->num_lines);
 #endif
 }
+
+/*winmesg.c*/
