@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)winstat.c	3.1	93/02/04	*/
+/*	SCCS Id: @(#)winstat.c	3.2	96/04/05	*/
 /* Copyright (c) Dean Luick, 1992				  */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -20,6 +20,7 @@
 #include <X11/Xaw/AsciiText.h>
 #include <X11/Xaw/Cardinals.h>
 #include <X11/Xaw/Form.h>
+#include <X11/Xaw/Paned.h>
 #include <X11/Xaw/Label.h>
 #include <X11/Xatom.h>
 
@@ -38,6 +39,7 @@ extern const char *enc_stat[]; /* from botl.c */
 
 static void FDECL(update_fancy_status, (struct xwindow *));
 static Widget FDECL(create_fancy_status, (Widget,Widget));
+static void FDECL(destroy_fancy_status, (struct xwindow *));
 
 void
 create_status_window(wp, create_popup, parent)
@@ -128,11 +130,18 @@ destroy_status_window(wp)
 {
     /* If status_information is defined, then it a "text" status window. */
     if (wp->status_information) {
-	nh_XtPopdown(wp->popup);
-	XtDestroyWidget(wp->popup);
-	free((char *) wp->status_information);
+	if (wp->popup) {
+	    nh_XtPopdown(wp->popup);
+	    if (!wp->keep_window)
+		XtDestroyWidget(wp->popup),  wp->popup = (Widget)0;
+	}
+	free((genericptr_t)wp->status_information);
+	wp->status_information = 0;
+    } else {
+	destroy_fancy_status(wp);
     }
-    wp->type = NHW_NONE;
+    if (!wp->keep_window)
+	wp->type = NHW_NONE;
 }
 
 
@@ -171,22 +180,13 @@ adjust_status(wp, str)
 
 
 /* Fancy Status -------------------------------------------------------------*/
-static Widget init_info_form();
-static Widget init_column();
-static void set_widths();
-static void get_widths();
-static void create_widget();
-static const char *width_string();
-static void hilight_label();
-static void update_val();
-
 static int hilight_time = 1;	/* number of turns to hilight a changed value */
 
 struct X_status_value {
     char    *name;		/* text name */
     int     type;		/* status type */
     Widget  w;			/* widget of name/value pair */
-    int     last_value;		/* value displayed */
+    long    last_value;		/* value displayed */
     int	    turn_count;		/* last time the value changed */
     boolean set;		/* if hilighed */
     boolean after_init;		/* don't hilight on first change (init) */
@@ -197,6 +197,15 @@ struct X_status_value {
 #define SV_LABEL 1	/* displays a changable label */
 #define SV_NAME  2	/* displays an unchangeable name */
 
+static void FDECL(hilight_label, (Widget));
+static void FDECL(update_val, (struct X_status_value *,long));
+static const char *FDECL(width_string, (int));
+static void FDECL(create_widget, (Widget,struct X_status_value *,int));
+static void FDECL(get_widths, (struct X_status_value *,int *,int *));
+static void FDECL(set_widths, (struct X_status_value *,int,int));
+static Widget FDECL(init_column, (char *,Widget,Widget,Widget,int *));
+static Widget FDECL(init_info_form, (Widget,Widget,Widget));
+
 /*
  * Form entry storage indices.
  */
@@ -261,7 +270,7 @@ static struct X_status_value shown_stats[NUM_STATS] = {
 
     { "",		SV_NAME,  (Widget) 0, -1, 0, FALSE, TRUE }, /* hunger*/
     { "Confused",	SV_NAME,  (Widget) 0,  0, 0, FALSE, TRUE },	/*20*/
-    { "Sick",		SV_NAME,  (Widget) 0,  0, 0, FALSE, TRUE },
+    { "",    		SV_NAME,  (Widget) 0,  0, 0, FALSE, TRUE }, /* sick */
     { "Blind",		SV_NAME,  (Widget) 0,  0, 0, FALSE, TRUE },
     { "Stunned",	SV_NAME,  (Widget) 0,  0, 0, FALSE, TRUE },
     { "Hallucinating",	SV_NAME,  (Widget) 0,  0, 0, FALSE, TRUE },
@@ -333,7 +342,6 @@ update_val(attr_rec, new_value)
 	    Strcpy(buf, plname);
 	    if ('a' <= buf[0] && buf[0] <= 'z') buf[0] += 'A'-'a';
 	    Strcat(buf, " the ");
-#ifdef POLYSELF
 	    if (u.mtimedone) {
 		char mname[BUFSZ];
 		int k = 0;
@@ -347,7 +355,6 @@ update_val(attr_rec, new_value)
 		}
 		Strcat(buf, mname);
 	    } else
-#endif
 		Strcat(buf, rank_of(u.ulevel, pl_character[0], flags.female));
 
 	} else if (attr_rec == &shown_stats[F_DLEVEL]) {
@@ -376,11 +383,23 @@ update_val(attr_rec, new_value)
 
 	attr_rec->last_value = new_value;
 
-	/* special cases: hunger and encumbrance */
+	/* special cases: hunger, encumbrance, sickness */
 	if (attr_rec == &shown_stats[F_HUNGER]) {
 	    XtSetArg(args[0], XtNlabel, hu_stat[new_value]);
 	} else if (attr_rec == &shown_stats[F_ENCUMBER]) {
 	    XtSetArg(args[0], XtNlabel, enc_stat[new_value]);
+	} else if (attr_rec == &shown_stats[F_SICK]) {
+	    buf[0] = 0;
+	    if (Sick) {
+		if (u.usick_type & SICK_VOMITABLE)
+		    Strcat(buf, "FoodPois");
+		if (u.usick_type & SICK_NONVOMITABLE) {
+		    if (u.usick_type & SICK_VOMITABLE)
+			Strcat(buf, " ");
+		    Strcat(buf, "Ill");
+		}
+	    }
+	    XtSetArg(args[0], XtNlabel, buf);
 	} else if (new_value) {
 	    XtSetArg(args[0], XtNlabel, attr_rec->name);
 	} else {
@@ -457,7 +476,6 @@ update_val(attr_rec, new_value)
 #endif
 	}
 
-#ifdef POLYSELF
 	/* special case: when polymorphed, show "HD", disable exp */
 	else if (attr_rec == &shown_stats[F_LEVEL]) {
 	    static boolean lev_was_poly = FALSE;
@@ -486,7 +504,6 @@ update_val(attr_rec, new_value)
 	    }
 	    if (u.mtimedone) return;	/* no display for exp when poly */
 	}
-#endif
 
 	if (attr_rec->last_value == new_value && !force_update)	/* same */
 	    return;
@@ -497,13 +514,13 @@ update_val(attr_rec, new_value)
 	if (attr_rec == &shown_stats[F_STR]) {
 	    if(new_value > 18) {
 		if (new_value > 118)
-		    Sprintf(buf,"%d", new_value-100);
+		    Sprintf(buf,"%ld", new_value-100);
 		else if(new_value < 118)
-		    Sprintf(buf, "18/%02d", new_value-18);
+		    Sprintf(buf, "18/%02ld", new_value-18);
 		else
 		    Strcpy(buf, "18/**");
 	    } else {
-		Sprintf(buf, "%d", new_value);
+		Sprintf(buf, "%ld", new_value);
 	    }
 	} else if (attr_rec == &shown_stats[F_ALIGN]) {
 
@@ -511,7 +528,7 @@ update_val(attr_rec, new_value)
 			(new_value == A_NEUTRAL) ? "Neutral" :
 						   "Lawful"  );
 	} else {
-	    Sprintf(buf, "%d", new_value);
+	    Sprintf(buf, "%ld", new_value);
 	}
 	set_value(attr_rec->w, buf);
     }
@@ -546,26 +563,21 @@ update_val(attr_rec, new_value)
  *	name, attributes, alignment, score
  *
  * Information on the second line:
- * 	dlvl, gold, hp, power, ac, {level & exp or HD **}
- * 	status (hunger, conf, halu, stun, sick, blind), time, encumbrance
+ *	dlvl, gold, hp, power, ac, {level & exp or HD **}
+ *	status (hunger, conf, halu, stun, sick, blind), time, encumbrance
  *
- * [**] HD is shown instead of level and exp if POLYSELF is defined and
- *	mtimedone is non-zero.
+ * [**] HD is shown instead of level and exp if mtimedone is non-zero.
  */
 static void
 update_fancy_status(wp)
     struct xwindow *wp;
 {
-    const struct X_status_value *sv;
+    struct X_status_value *sv;
     long val;
     int i;
 
     if (wp->cursy != 0) return;	/* do a complete update when line 0 is done */
 
-#ifdef GCC_WARN
-    val = 0;
-#endif
-
     for (i = 0, sv = shown_stats; i < NUM_STATS; i++, sv++) {
 	switch (i) {
 	    case F_STR:		val = (long) ACURR(A_STR); break;
@@ -575,13 +587,14 @@ update_fancy_status(wp)
 	    case F_WIS:		val = (long) ACURR(A_WIS); break;
 	    case F_CHA:		val = (long) ACURR(A_CHA); break;
 	    /*
-	     * Label stats.  With the exceptions of hunger and encumbrance,
+	     * Label stats.  With the exceptions of hunger, encumbrance, sick
 	     * these are either on or off.  Pleae leave the ternary operators
 	     * the way they are.  I want to specify 0 or 1, not a boolean.
 	     */
 	    case F_HUNGER:	val = (long) u.uhs;			break;
 	    case F_CONFUSED:	val = (long) Confusion     ? 1L : 0L;	break;
-	    case F_SICK:	val = (long) Sick	   ? 1L : 0L;	break;
+	    case F_SICK:	val = (long) Sick ? (long)u.usick_type
+								: 0L;	break;
 	    case F_BLIND:	val = (long) Blind	   ? 1L : 0L;	break;
 	    case F_STUNNED:	val = (long) Stunned	   ? 1L : 0L;	break;
 	    case F_HALLU:	val = (long) Hallucination ? 1L : 0L;	break;
@@ -590,26 +603,17 @@ update_fancy_status(wp)
 	    case F_NAME:	val = (long) 0L; break;	/* special */
 	    case F_DLEVEL:	val = (long) 0L; break;	/* special */
 	    case F_GOLD:	val = (long) u.ugold; break;
-#ifdef POLYSELF
 	    case F_HP:		val = (long) (u.mtimedone ?
 					      (u.mh  > 0 ? u.mh  : 0):
 					      (u.uhp > 0 ? u.uhp : 0)); break;
 	    case F_MAXHP:	val = (long) (u.mtimedone ? u.mhmax :
 							    u.uhpmax);  break;
-#else
-	    case F_HP:		val = (long) (u.uhp > 0 ? u.uhp : 0);	break;
-	    case F_MAXHP:	val = (long) u.uhpmax;	break;
-#endif
 	    case F_POWER:	val = (long) u.uen;	break;
 	    case F_MAXPOWER:	val = (long) u.uenmax;	break;
 	    case F_AC:		val = (long) u.uac;	break;
-#ifdef POLYSELF
 	    case F_LEVEL:	val = (long) (u.mtimedone ?
 						mons[u.umonnum].mlevel :
 						u.ulevel);		break;
-#else
-	    case F_LEVEL:	val = (long) u.ulevel;	break;
-#endif
 #ifdef EXP_ON_BOTL
 	    case F_EXP:		val = flags.showexp ? u.uexp : 0L; break;
 #else
@@ -638,6 +642,7 @@ update_fancy_status(wp)
 		    impossible("update_other: unknown shown value");
 		    active = FALSE;
 		}
+		val = 0;
 		break;
 	    }
 	}
@@ -908,7 +913,7 @@ init_info_form(parent, top, left)
     create_widget(form, sv_dlevel, F_DLEVEL);
 
     num_args = 0;
-    XtSetArg(args[num_args], XtNfromVert, sv_name->w); 	num_args++;
+    XtSetArg(args[num_args], XtNfromVert, sv_name->w);	num_args++;
     XtSetValues(sv_dlevel->w, args, num_args);
 
     /* two columns beneath */
@@ -945,7 +950,7 @@ create_fancy_status(parent, top)
 {
     Widget form;	/* The form that surrounds everything. */
     Widget w;
-    Arg args[6];
+    Arg args[8];
     Cardinal num_args;
 
     num_args = 0;
@@ -954,8 +959,9 @@ create_fancy_status(parent, top)
     }
     XtSetArg(args[num_args], XtNdefaultDistance, 0);	num_args++;
     XtSetArg(args[num_args], XtNborderWidth, 0);	num_args++;
+    XtSetArg(args[num_args], XtNorientation, XtorientHorizontal); num_args++;
     form = XtCreateManagedWidget("fancy_status",
-				formWidgetClass,
+				panedWidgetClass,
 				parent,
 				args, num_args);
 
@@ -965,3 +971,21 @@ create_fancy_status(parent, top)
     return form;
 }
 
+static void
+destroy_fancy_status(wp)
+struct xwindow *wp;
+{
+    int i;
+    struct X_status_value *sv;
+
+    if (!wp->keep_window)
+	XtDestroyWidget(wp->w),  wp->w = (Widget)0;
+
+    for (i = 0, sv = shown_stats; i < NUM_STATS; i++, sv++)
+	if (sv->type == SV_LABEL) {
+	    free((genericptr_t)sv->name);
+	    sv->name = 0;
+	}
+}
+
+/*winstat.c*/
